"""
Tests of regrtest.py.

Note: test_regrtest cannot be run twice in parallel.
"""
from __future__ import print_function

import collections
import errno
import os.path
import platform
import re
import subprocess
import sys
import sysconfig
import tempfile
import textwrap
import unittest
from test import support
# Use utils alias to use the same code for TestUtils in master and 2.7 branches
import regrtest as utils


Py_DEBUG = hasattr(sys, 'getobjects')
ROOT_DIR = os.path.join(os.path.dirname(__file__), '..', '..')
ROOT_DIR = os.path.abspath(os.path.normpath(ROOT_DIR))

TEST_INTERRUPTED = textwrap.dedent("""
    from signal import SIGINT
    try:
        from _testcapi import raise_signal
        raise_signal(SIGINT)
    except ImportError:
        import os
        os.kill(os.getpid(), SIGINT)
    """)


SubprocessRun = collections.namedtuple('SubprocessRun',
                                       'returncode stdout stderr')


class BaseTestCase(unittest.TestCase):
    TEST_UNIQUE_ID = 1
    TESTNAME_PREFIX = 'test_regrtest_'
    TESTNAME_REGEX = r'test_[a-zA-Z0-9_]+'

    def setUp(self):
        self.testdir = os.path.realpath(os.path.dirname(__file__))

        self.tmptestdir = tempfile.mkdtemp()
        self.addCleanup(support.rmtree, self.tmptestdir)

    def create_test(self, name=None, code=None):
        if not name:
            name = 'noop%s' % BaseTestCase.TEST_UNIQUE_ID
            BaseTestCase.TEST_UNIQUE_ID += 1

        if code is None:
            code = textwrap.dedent("""
                    import unittest
                    from test import support

                    class Tests(unittest.TestCase):
                        def test_empty_test(self):
                            pass

                    def test_main():
                        support.run_unittest(Tests)
                """)

        # test_regrtest cannot be run twice in parallel because
        # of setUp() and create_test()
        name = self.TESTNAME_PREFIX + name
        path = os.path.join(self.tmptestdir, name + '.py')

        self.addCleanup(support.unlink, path)
        # Use O_EXCL to ensure that we do not override existing tests
        try:
            fd = os.open(path, os.O_WRONLY | os.O_CREAT | os.O_EXCL)
        except OSError as exc:
            if (exc.errno in (errno.EACCES, errno.EPERM)
               and not sysconfig.is_python_build()):
                self.skipTest("cannot write %s: %s" % (path, exc))
            else:
                raise
        else:
            with os.fdopen(fd, 'w') as fp:
                fp.write(code)
        return name

    def regex_search(self, regex, output):
        match = re.search(regex, output, re.MULTILINE)
        if not match:
            self.fail("%r not found in %r" % (regex, output))
        return match

    def check_line(self, output, regex):
        regex = re.compile(r'^' + regex, re.MULTILINE)
        self.assertRegexpMatches(output, regex)

    def parse_executed_tests(self, output):
        regex = (r'^[0-9]+:[0-9]+:[0-9]+ (?:load avg: [0-9]+\.[0-9]{2} )?\[ *[0-9]+(?:/ *[0-9]+)*\] (%s)'
                 % self.TESTNAME_REGEX)
        parser = re.finditer(regex, output, re.MULTILINE)
        return list(match.group(1) for match in parser)

    def check_executed_tests(self, output, tests, skipped=(), failed=(),
                             env_changed=(), omitted=(),
                             rerun=(), no_test_ran=(),
                             randomize=False, interrupted=False,
                             fail_env_changed=False):
        if isinstance(tests, str):
            tests = [tests]
        if isinstance(skipped, str):
            skipped = [skipped]
        if isinstance(failed, str):
            failed = [failed]
        if isinstance(env_changed, str):
            env_changed = [env_changed]
        if isinstance(omitted, str):
            omitted = [omitted]
        if isinstance(rerun, str):
            rerun = [rerun]
        if isinstance(no_test_ran, str):
            no_test_ran = [no_test_ran]

        executed = self.parse_executed_tests(output)
        if randomize:
            self.assertEqual(set(executed), set(tests), output)
        else:
            self.assertEqual(executed, tests, (executed, tests, output))

        def plural(count):
            return 's' if count != 1 else ''

        def list_regex(line_format, tests):
            count = len(tests)
            names = ' '.join(sorted(tests))
            regex = line_format % (count, plural(count))
            regex = r'%s:\n    %s$' % (regex, names)
            return regex

        if skipped:
            regex = list_regex('%s test%s skipped', skipped)
            self.check_line(output, regex)

        if failed:
            regex = list_regex('%s test%s failed', failed)
            self.check_line(output, regex)

        if env_changed:
            regex = list_regex('%s test%s altered the execution environment',
                               env_changed)
            self.check_line(output, regex)

        if omitted:
            regex = list_regex('%s test%s omitted', omitted)
            self.check_line(output, regex)

        if rerun:
            regex = list_regex('%s re-run test%s', rerun)
            self.check_line(output, regex)
            self.check_line(output, "Re-running failed tests in verbose mode")
            for name in rerun:
                regex = "Re-running test %r in verbose mode" % name
                self.check_line(output, regex)

        good = (len(tests) - len(skipped) - len(failed)
                - len(omitted) - len(env_changed) - len(no_test_ran))
        if good:
            regex = r'%s test%s OK\.$' % (good, plural(good))
            if not skipped and not failed and good > 1:
                regex = 'All %s' % regex
            self.check_line(output, regex)

        if interrupted:
            self.check_line(output, 'Test suite interrupted by signal SIGINT.')

        result = []
        if failed:
            result.append('FAILURE')
        elif fail_env_changed and env_changed:
            result.append('ENV CHANGED')
        if interrupted:
            result.append('INTERRUPTED')
        if not any((good, result, failed, interrupted, skipped,
                    env_changed, fail_env_changed)):
            result.append("NO TEST RUN")
        elif not result:
            result.append('SUCCESS')
        result = ', '.join(result)
        if rerun:
            self.check_line(output, 'Tests result: %s' % result)
            result = 'FAILURE then %s' % result

        self.check_line(output, 'Tests result: %s' % result)

    def parse_random_seed(self, output):
        match = self.regex_search(r'Using random seed ([0-9]+)', output)
        randseed = int(match.group(1))
        self.assertTrue(0 <= randseed <= 10000000, randseed)
        return randseed

    def run_command(self, args, input=None, exitcode=0, **kw):
        if not input:
            input = ''
        if 'stderr' not in kw:
            kw['stderr'] = subprocess.PIPE
        proc = subprocess.Popen(args,
                                universal_newlines=True,
                                stdout=subprocess.PIPE,
                                **kw)
        stdout, stderr = proc.communicate(input=input)
        if proc.returncode != exitcode:
            msg = ("Command %s failed with exit code %s\n"
                   "\n"
                   "stdout:\n"
                   "---\n"
                   "%s\n"
                   "---\n"
                   % (str(args), proc.returncode, stdout))
            if proc.stderr:
                msg += ("\n"
                        "stderr:\n"
                        "---\n"
                        "%s"
                        "---\n"
                        % stderr)
            self.fail(msg)
        return SubprocessRun(proc.returncode, stdout, stderr)

    def run_python(self, args, **kw):
        args = [sys.executable] + list(args)
        proc = self.run_command(args, **kw)
        return proc.stdout


class ProgramsTestCase(BaseTestCase):
    """
    Test various ways to run the Python test suite. Use options close
    to options used on the buildbot.
    """

    NTEST = 4

    def setUp(self):
        super(ProgramsTestCase, self).setUp()

        # Create NTEST tests doing nothing
        self.tests = [self.create_test() for index in range(self.NTEST)]

        self.python_args = ['-Wd', '-3', '-E', '-bb', '-tt']
        self.regrtest_args = ['-uall', '-rwW',
                              '--testdir=%s' % self.tmptestdir]

    def check_output(self, output):
        self.parse_random_seed(output)
        self.check_executed_tests(output, self.tests, randomize=True)

    def run_tests(self, args):
        output = self.run_python(args)
        self.check_output(output)

    def test_script_regrtest(self):
        # Lib/test/regrtest.py
        script = os.path.join(self.testdir, 'regrtest.py')

        args = self.python_args + [script] + self.regrtest_args + self.tests
        self.run_tests(args)

    def test_module_test(self):
        # -m test
        args = self.python_args + ['-m', 'test'] + self.regrtest_args + self.tests
        self.run_tests(args)

    def test_module_regrtest(self):
        # -m test.regrtest
        args = self.python_args + ['-m', 'test.regrtest'] + self.regrtest_args + self.tests
        self.run_tests(args)

    def test_module_autotest(self):
        # -m test.autotest
        args = self.python_args + ['-m', 'test.autotest'] + self.regrtest_args + self.tests
        self.run_tests(args)

    def test_module_from_test_autotest(self):
        # from test import autotest
        code = 'from test import autotest'
        args = self.python_args + ['-c', code] + self.regrtest_args + self.tests
        self.run_tests(args)

    def test_script_autotest(self):
        # Lib/test/autotest.py
        script = os.path.join(self.testdir, 'autotest.py')
        args = self.python_args + [script] + self.regrtest_args + self.tests
        self.run_tests(args)

    def run_batch(self, *args):
        proc = self.run_command(args)
        self.check_output(proc.stdout)

    def need_pcbuild(self):
        exe = os.path.normpath(os.path.abspath(sys.executable))
        parts = exe.split(os.path.sep)
        if len(parts) < 3:
            # it's not a python build, python is likely to be installed
            return

        build_dir = parts[-3]
        if build_dir.lower() != 'pcbuild':
            self.skipTest("Tools/buildbot/test.bat requires PCbuild build, "
                          "found %s" % build_dir)

    @unittest.skipUnless(sysconfig.is_python_build(),
                         'test.bat script is not installed')
    @unittest.skipUnless(sys.platform == 'win32', 'Windows only')
    def test_tools_buildbot_test(self):
        self.need_pcbuild()

        # Tools\buildbot\test.bat
        script = os.path.join(ROOT_DIR, 'Tools', 'buildbot', 'test.bat')
        test_args = ['--testdir=%s' % self.tmptestdir]
        if platform.architecture()[0] == '64bit':
            test_args.append('-x64')   # 64-bit build
        if not Py_DEBUG:
            test_args.append('+d')     # Release build, use python.exe

        args = [script] + test_args + self.tests
        self.run_batch(*args)

    @unittest.skipUnless(sys.platform == 'win32', 'Windows only')
    def test_pcbuild_rt(self):
        self.need_pcbuild()

        # PCbuild\rt.bat
        script = os.path.join(ROOT_DIR, r'PCbuild\rt.bat')
        rt_args = ["-q"]             # Quick, don't run tests twice
        if platform.architecture()[0] == '64bit':
            rt_args.append('-x64')   # 64-bit build
        if Py_DEBUG:
            rt_args.append('-d')     # Debug build, use python_d.exe
        args = [script] + rt_args + self.regrtest_args + self.tests
        self.run_batch(*args)


class ArgsTestCase(BaseTestCase):
    """
    Test arguments of the Python test suite.
    """

    def run_tests(self, *testargs, **kw):
        cmdargs = ('-m', 'test', '--testdir=%s' % self.tmptestdir) + testargs
        return self.run_python(cmdargs, **kw)

    def test_failing_test(self):
        # test a failing test
        code = textwrap.dedent("""
            import unittest
            from test import support

            class FailingTest(unittest.TestCase):
                def test_failing(self):
                    self.fail("bug")

            def test_main():
                support.run_unittest(FailingTest)
        """)
        test_ok = self.create_test('ok')
        test_failing = self.create_test('failing', code=code)
        tests = [test_ok, test_failing]

        output = self.run_tests(*tests, exitcode=2)
        self.check_executed_tests(output, tests, failed=test_failing)

    def test_resources(self):
        # test -u command line option
        tests = {}
        for resource in ('audio', 'network'):
            code = textwrap.dedent("""
                        from test import support; support.requires(%r)
                        import unittest
                        class PassingTest(unittest.TestCase):
                            def test_pass(self):
                                pass

                        def test_main():
                            support.run_unittest(PassingTest)
                    """ % resource)

            tests[resource] = self.create_test(resource, code)
        test_names = sorted(tests.values())

        # -u all: 2 resources enabled
        output = self.run_tests('-u', 'all', *test_names)
        self.check_executed_tests(output, test_names)

        # -u audio: 1 resource enabled
        output = self.run_tests('-uaudio', *test_names)
        self.check_executed_tests(output, test_names,
                                  skipped=tests['network'])

        # no option: 0 resources enabled
        output = self.run_tests(*test_names)
        self.check_executed_tests(output, test_names,
                                  skipped=test_names)

    def test_random(self):
        # test -r and --randseed command line option
        code = textwrap.dedent("""
            import random
            print("TESTRANDOM: %s" % random.randint(1, 1000))
        """)
        test = self.create_test('random', code)

        # first run to get the output with the random seed
        output = self.run_tests('-r', '-v', test)
        randseed = self.parse_random_seed(output)
        match = self.regex_search(r'TESTRANDOM: ([0-9]+)', output)
        test_random = int(match.group(1))

        # try to reproduce with the random seed
        output = self.run_tests('-r', '-v', '--randseed=%s' % randseed, test)
        randseed2 = self.parse_random_seed(output)
        self.assertEqual(randseed2, randseed)

        match = self.regex_search(r'TESTRANDOM: ([0-9]+)', output)
        test_random2 = int(match.group(1))
        self.assertEqual(test_random2, test_random)

    def test_fromfile(self):
        # test --fromfile
        tests = [self.create_test() for index in range(5)]

        # Write the list of files using a format similar to regrtest output:
        # [1/2] test_1
        # [2/2] test_2
        filename = support.TESTFN
        self.addCleanup(support.unlink, filename)

        # test format 'test_opcodes'
        with open(filename, "w") as fp:
            for name in tests:
                print(name, file=fp)

        output = self.run_tests('--fromfile', filename)
        self.check_executed_tests(output, tests)

    def test_interrupted(self):
        code = TEST_INTERRUPTED
        test = self.create_test('sigint', code=code)
        output = self.run_tests(test, exitcode=130)
        self.check_executed_tests(output, test, omitted=test,
                                  interrupted=True)

    def test_slowest(self):
        # test --slow
        tests = [self.create_test() for index in range(3)]
        output = self.run_tests("--slowest", *tests)
        self.check_executed_tests(output, tests)
        regex = ('10 slowest tests:\n'
                 '(?:- %s: .*\n){%s}'
                 % (self.TESTNAME_REGEX, len(tests)))
        self.check_line(output, regex)

    def test_slow_interrupted(self):
        # Issue #25373: test --slowest with an interrupted test
        code = TEST_INTERRUPTED
        test = self.create_test("sigint", code=code)

        try:
            import threading
            tests = (False, True)
        except ImportError:
            tests = (False,)
        for multiprocessing in tests:
            if multiprocessing:
                args = ("--slowest", "-j2", test)
            else:
                args = ("--slowest", test)
            output = self.run_tests(*args, exitcode=130)
            self.check_executed_tests(output, test,
                                      omitted=test, interrupted=True)

            regex = ('10 slowest tests:\n')
            self.check_line(output, regex)

    def test_coverage(self):
        # test --coverage
        test = self.create_test('coverage')
        output = self.run_tests("--coverage", test)
        self.check_executed_tests(output, [test])
        regex = (r'lines +cov% +module +\(path\)\n'
                 r'(?: *[0-9]+ *[0-9]{1,2}% *[^ ]+ +\([^)]+\)+)+')
        self.check_line(output, regex)

    def test_forever(self):
        # test --forever
        code = textwrap.dedent("""
            import __builtin__
            import unittest
            from test import support

            class ForeverTester(unittest.TestCase):
                def test_run(self):
                    # Store the state in the __builtin__ module, because the test
                    # module is reload at each run
                    if 'RUN' in __builtin__.__dict__:
                        __builtin__.__dict__['RUN'] += 1
                        if __builtin__.__dict__['RUN'] >= 3:
                            self.fail("fail at the 3rd runs")
                    else:
                        __builtin__.__dict__['RUN'] = 1

            def test_main():
                support.run_unittest(ForeverTester)
        """)
        test = self.create_test('forever', code=code)
        output = self.run_tests('--forever', test, exitcode=2)
        self.check_executed_tests(output, [test]*3, failed=test)

    def check_leak(self, code, what):
        test = self.create_test('huntrleaks', code=code)

        filename = 'reflog.txt'
        self.addCleanup(support.unlink, filename)
        output = self.run_tests('--huntrleaks', '3:3:', test,
                                exitcode=2,
                                stderr=subprocess.STDOUT)
        self.check_executed_tests(output, [test], failed=test)

        line = 'beginning 6 repetitions\n123456\n......\n'
        self.check_line(output, re.escape(line))

        line2 = '%s leaked [1, 1, 1] %s, sum=3\n' % (test, what)
        self.assertIn(line2, output)

        with open(filename) as fp:
            reflog = fp.read()
            self.assertIn(line2, reflog)

    @unittest.skipUnless(Py_DEBUG, 'need a debug build')
    @support.requires_type_collecting
    def test_huntrleaks(self):
        # test --huntrleaks
        code = textwrap.dedent("""
            import unittest
            from test import support

            GLOBAL_LIST = []

            class RefLeakTest(unittest.TestCase):
                def test_leak(self):
                    GLOBAL_LIST.append(object())

            def test_main():
                support.run_unittest(RefLeakTest)
        """)
        self.check_leak(code, 'references')

    @unittest.skipUnless(Py_DEBUG, 'need a debug build')
    def test_huntrleaks_fd_leak(self):
        # test --huntrleaks for file descriptor leak
        code = textwrap.dedent("""
            import os
            import unittest
            from test import support

            class FDLeakTest(unittest.TestCase):
                def test_leak(self):
                    fd = os.open(__file__, os.O_RDONLY)
                    # bug: never close the file descriptor

            def test_main():
                support.run_unittest(FDLeakTest)
        """)
        self.check_leak(code, 'file descriptors')

    def test_list_tests(self):
        # test --list-tests
        tests = [self.create_test() for i in range(5)]
        output = self.run_tests('--list-tests', *tests)
        self.assertEqual(output.rstrip().splitlines(),
                         tests)

    def test_list_cases(self):
        # test --list-cases
        code = textwrap.dedent("""
            import unittest

            class Tests(unittest.TestCase):
                def test_method1(self):
                    pass
                def test_method2(self):
                    pass
        """)
        testname = self.create_test(code=code)

        # Test --list-cases
        all_methods = ['%s.Tests.test_method1' % testname,
                       '%s.Tests.test_method2' % testname]
        output = self.run_tests('--list-cases', testname)
        self.assertEqual(output.splitlines(), all_methods)

        # Test --list-cases with --match
        all_methods = ['%s.Tests.test_method1' % testname]
        output = self.run_tests('--list-cases',
                                '-m', 'test_method1',
                                testname)
        self.assertEqual(output.splitlines(), all_methods)

    @unittest.skipIf(sys.platform.startswith('aix'),
                     "support._crash_python() doesn't work on AIX")
    def test_crashed(self):
        # Any code which causes a crash
        code = 'import test.support; test.support._crash_python()'
        crash_test = self.create_test(name="crash", code=code)
        ok_test = self.create_test(name="ok")

        tests = [crash_test, ok_test]
        output = self.run_tests("-j2", *tests, exitcode=2)
        self.check_executed_tests(output, tests, failed=crash_test,
                                  randomize=True)

    def parse_methods(self, output):
        regex = re.compile("^(test[^ ]+).*ok$", flags=re.MULTILINE)
        return [match.group(1) for match in regex.finditer(output)]

    def test_matchfile(self):
        # Any code which causes a crash
        code = textwrap.dedent("""
            import unittest
            from test import support

            class Tests(unittest.TestCase):
                def test_method1(self):
                    pass
                def test_method2(self):
                    pass
                def test_method3(self):
                    pass
                def test_method4(self):
                    pass

            def test_main():
                support.run_unittest(Tests)
        """)
        all_methods = ['test_method1', 'test_method2',
                       'test_method3', 'test_method4']
        testname = self.create_test(code=code)

        # by default, all methods should be run
        output = self.run_tests("-v", testname)
        methods = self.parse_methods(output)
        self.assertEqual(methods, all_methods)

        # only run a subset
        filename = support.TESTFN
        self.addCleanup(support.unlink, filename)

        subset = [
            # only match the method name
            'test_method1',
            # match the full identifier
            '%s.Tests.test_method3' % testname]
        with open(filename, "w") as fp:
            for name in subset:
                print(name, file=fp)

        output = self.run_tests("-v", "--matchfile", filename, testname)
        methods = self.parse_methods(output)
        subset = ['test_method1', 'test_method3']
        self.assertEqual(methods, subset)

    # bpo-34021: The test fails randomly for an unknown reason
    # on "x86 Windows XP VS9.0 2.7" buildbot worker.
    @unittest.skipIf(sys.platform == "win32", "test fails randomly on Windows")
    def test_env_changed(self):
        code = textwrap.dedent("""
            import unittest
            from test import support

            class Tests(unittest.TestCase):
                def test_env_changed(self):
                    open("env_changed", "w").close()

            def test_main():
                support.run_unittest(Tests)
        """)
        testname = self.create_test(code=code)

        # don't fail by default
        output = self.run_tests(testname)
        self.check_executed_tests(output, [testname], env_changed=testname)

        # fail with --fail-env-changed
        output = self.run_tests("--fail-env-changed", testname, exitcode=3)
        self.check_executed_tests(output, [testname], env_changed=testname,
                                  fail_env_changed=True)

    def test_rerun_fail(self):
        code = textwrap.dedent("""
            import unittest
            from test import support

            class Tests(unittest.TestCase):
                def test_bug(self):
                    # test always fail
                    self.fail("bug")

            def test_main():
                support.run_unittest(Tests)
        """)
        testname = self.create_test(code=code)

        output = self.run_tests("-w", testname, exitcode=2)
        self.check_executed_tests(output, [testname],
                                  failed=testname, rerun=testname)

    def test_no_tests_ran(self):
        code = textwrap.dedent("""
            import unittest
            from test import support

            class Tests(unittest.TestCase):
                def test_bug(self):
                    pass

            def test_main():
                support.run_unittest(Tests)
        """)
        testname = self.create_test(code=code)

        output = self.run_tests("-m", "nosuchtest", testname, exitcode=0)
        self.check_executed_tests(output, [testname], no_test_ran=testname)

    def test_no_tests_ran_skip(self):
        code = textwrap.dedent("""
            import unittest

            class Tests(unittest.TestCase):
                def test_skipped(self):
                    self.skipTest("because")
        """)
        testname = self.create_test(code=code)

        output = self.run_tests(testname, exitcode=0)
        self.check_executed_tests(output, [testname])

    def test_no_tests_ran_multiple_tests_nonexistent(self):
        code = textwrap.dedent("""
            import unittest
            from test import support

            class Tests(unittest.TestCase):
                def test_bug(self):
                    pass

            def test_main():
                support.run_unittest(Tests)
        """)
        testname = self.create_test(code=code)
        testname2 = self.create_test(code=code)

        output = self.run_tests("-m", "nosuchtest",
                                testname, testname2,
                                exitcode=0)
        self.check_executed_tests(output, [testname, testname2],
                                  no_test_ran=[testname, testname2])

    def test_no_test_ran_some_test_exist_some_not(self):
        code = textwrap.dedent("""
            import unittest
            from test import support

            class Tests(unittest.TestCase):
                def test_bug(self):
                    pass

            def test_main():
                support.run_unittest(Tests)
        """)
        testname = self.create_test(code=code)
        other_code = textwrap.dedent("""
            import unittest
            from test import support

            class Tests(unittest.TestCase):
                def test_other_bug(self):
                    pass

            def test_main():
                support.run_unittest(Tests)
        """)
        testname2 = self.create_test(code=other_code)

        output = self.run_tests("-m", "nosuchtest", "-m", "test_other_bug",
                                testname, testname2,
                                exitcode=0)
        self.check_executed_tests(output, [testname, testname2],
                                  no_test_ran=[testname])


class TestUtils(unittest.TestCase):
    def test_format_duration(self):
        self.assertEqual(utils.format_duration(0),
                         '0 ms')
        self.assertEqual(utils.format_duration(1e-9),
                         '1 ms')
        self.assertEqual(utils.format_duration(10e-3),
                         '10 ms')
        self.assertEqual(utils.format_duration(1.5),
                         '1 sec 500 ms')
        self.assertEqual(utils.format_duration(1),
                         '1 sec')
        self.assertEqual(utils.format_duration(2 * 60),
                         '2 min')
        self.assertEqual(utils.format_duration(2 * 60 + 1),
                         '2 min 1 sec')
        self.assertEqual(utils.format_duration(3 * 3600),
                         '3 hour')
        self.assertEqual(utils.format_duration(3 * 3600  + 2 * 60 + 1),
                         '3 hour 2 min')
        self.assertEqual(utils.format_duration(3 * 3600 + 1),
                         '3 hour 1 sec')


def test_main():
    support.run_unittest(ProgramsTestCase, ArgsTestCase, TestUtils)


if __name__ == "__main__":
    test_main()



# Generated from 'CGContext.h'

def FOUR_CHAR_CODE(x): return x
kCGLineJoinMiter = 0
kCGLineJoinRound = 1
kCGLineJoinBevel = 2
kCGLineCapButt = 0
kCGLineCapRound = 1
kCGLineCapSquare = 2
kCGPathFill = 0
kCGPathEOFill = 1
kCGPathStroke = 2
kCGPathFillStroke = 3
kCGPathEOFillStroke = 4
kCGTextFill = 0
kCGTextStroke = 1
kCGTextFillStroke = 2
kCGTextInvisible = 3
kCGTextFillClip = 4
kCGTextStrokeClip = 5
kCGTextFillStrokeClip = 6
kCGTextClip = 7
kCGEncodingFontSpecific = 0
kCGEncodingMacRoman = 1
kCGInterpolationDefault = 0
kCGInterpolationNone = 1
kCGInterpolationLow = 2
kCGInterpolationHigh = 3



"A sort of application framework for the Mac"

DEBUG=0

from warnings import warnpy3k
warnpy3k("In 3.x, the FrameWork module is removed.", stacklevel=2)

import MacOS
import traceback

from Carbon.AE import *
from Carbon.AppleEvents import *
from Carbon.Ctl import *
from Carbon.Controls import *
from Carbon.Dlg import *
from Carbon.Dialogs import *
from Carbon.Evt import *
from Carbon.Events import *
from Carbon.Help import *
from Carbon.Menu import *
from Carbon.Menus import *
from Carbon.Qd import *
from Carbon.QuickDraw import *
#from Carbon.Res import *
#from Carbon.Resources import *
#from Carbon.Snd import *
#from Carbon.Sound import *
from Carbon.Win import *
from Carbon.Windows import *
import types

import EasyDialogs

try:
    MyFrontWindow = FrontNonFloatingWindow
except NameError:
    MyFrontWindow = FrontWindow

kHighLevelEvent = 23    # Don't know what header file this should come from
SCROLLBARWIDTH = 16         # Again, not a clue...

# Trick to forestall a set of SIOUX menus being added to our menubar
SIOUX_APPLEMENU_ID=32000


# Map event 'what' field to strings
eventname = {}
eventname[1] = 'mouseDown'
eventname[2] = 'mouseUp'
eventname[3] = 'keyDown'
eventname[4] = 'keyUp'
eventname[5] = 'autoKey'
eventname[6] = 'updateEvt'
eventname[7] = 'diskEvt'
eventname[8] = 'activateEvt'
eventname[15] = 'osEvt'
eventname[23] = 'kHighLevelEvent'

# Map part codes returned by WhichWindow() to strings
partname = {}
partname[0] = 'inDesk'
partname[1] = 'inMenuBar'
partname[2] = 'inSysWindow'
partname[3] = 'inContent'
partname[4] = 'inDrag'
partname[5] = 'inGrow'
partname[6] = 'inGoAway'
partname[7] = 'inZoomIn'
partname[8] = 'inZoomOut'

#
# The useable portion of the screen
#       ## but what happens with multiple screens? jvr
screenbounds = GetQDGlobalsScreenBits().bounds
screenbounds = screenbounds[0]+4, screenbounds[1]+4, \
    screenbounds[2]-4, screenbounds[3]-4

next_window_x = 16          # jvr
next_window_y = 44          # jvr

def windowbounds(width, height):
    "Return sensible window bounds"
    global next_window_x, next_window_y
    r, b = next_window_x+width, next_window_y+height
    if r > screenbounds[2]:
        next_window_x = 16
    if b > screenbounds[3]:
        next_window_y = 44
    l, t = next_window_x, next_window_y
    r, b = next_window_x+width, next_window_y+height
    next_window_x, next_window_y = next_window_x + 8, next_window_y + 20    # jvr
    return l, t, r, b

_watch = None
def setwatchcursor():
    global _watch

    if _watch is None:
        _watch = GetCursor(4).data
    SetCursor(_watch)

def setarrowcursor():
    SetCursor(GetQDGlobalsArrow())

class Application:

    "Application framework -- your application should be a derived class"

    def __init__(self, nomenubar=0):
        self._doing_asyncevents = 0
        self.quitting = 0
        self.needmenubarredraw = 0
        self._windows = {}
        self._helpmenu = None
        if nomenubar:
            self.menubar = None
        else:
            self.makemenubar()

    def __del__(self):
        if self._doing_asyncevents:
            self._doing_asyncevents = 0
            MacOS.SetEventHandler()

    def makemenubar(self):
        self.menubar = MenuBar(self)
        AppleMenu(self.menubar, self.getabouttext(), self.do_about)
        self.makeusermenus()

    def makeusermenus(self):
        self.filemenu = m = Menu(self.menubar, "File")
        self._quititem = MenuItem(m, "Quit", "Q", self._quit)

    def gethelpmenu(self):
        if self._helpmenu is None:
            self._helpmenu = HelpMenu(self.menubar)
        return self._helpmenu

    def _quit(self, *args):
        self.quitting = 1

    def cleanup(self):
        for w in self._windows.values():
            w.do_close()
        return self._windows == {}

    def appendwindow(self, wid, window):
        self._windows[wid] = window

    def removewindow(self, wid):
        del self._windows[wid]

    def getabouttext(self):
        return "About %s..." % self.__class__.__name__

    def do_about(self, id, item, window, event):
        EasyDialogs.Message("Hello, world!" + "\015(%s)" % self.__class__.__name__)

    # The main event loop is broken up in several simple steps.
    # This is done so you can override each individual part,
    # if you have a need to do extra processing independent of the
    # event type.
    # Normally, however, you'd just define handlers for individual
    # events.

    schedparams = (0, 0)    # By default disable Python's event handling
    default_wait = None         # By default we wait GetCaretTime in WaitNextEvent

    def mainloop(self, mask = everyEvent, wait = None):
        self.quitting = 0
        if hasattr(MacOS, 'SchedParams'):
            saveparams = MacOS.SchedParams(*self.schedparams)
        try:
            while not self.quitting:
                try:
                    self.do1event(mask, wait)
                except (Application, SystemExit):
                    # Note: the raising of "self" is old-fashioned idiom to
                    # exit the mainloop. Calling _quit() is better for new
                    # applications.
                    break
        finally:
            if hasattr(MacOS, 'SchedParams'):
                MacOS.SchedParams(*saveparams)

    def dopendingevents(self, mask = everyEvent):
        """dopendingevents - Handle all pending events"""
        while self.do1event(mask, wait=0):
            pass

    def do1event(self, mask = everyEvent, wait = None):
        ok, event = self.getevent(mask, wait)
        if IsDialogEvent(event):
            if self.do_dialogevent(event):
                return
        if ok:
            self.dispatch(event)
        else:
            self.idle(event)

    def idle(self, event):
        pass

    def getevent(self, mask = everyEvent, wait = None):
        if self.needmenubarredraw:
            DrawMenuBar()
            self.needmenubarredraw = 0
        if wait is None:
            wait = self.default_wait
            if wait is None:
                wait = GetCaretTime()
        ok, event = WaitNextEvent(mask, wait)
        return ok, event

    def dispatch(self, event):
        # The following appears to be double work (already done in do1event)
        # but we need it for asynchronous event handling
        if IsDialogEvent(event):
            if self.do_dialogevent(event):
                return
        (what, message, when, where, modifiers) = event
        if what in eventname:
            name = "do_" + eventname[what]
        else:
            name = "do_%d" % what
        try:
            handler = getattr(self, name)
        except AttributeError:
            handler = self.do_unknownevent
        handler(event)

    def asyncevents(self, onoff):
        """asyncevents - Set asynchronous event handling on or off"""
        if MacOS.runtimemodel == 'macho':
            raise 'Unsupported in MachoPython'
        old = self._doing_asyncevents
        if old:
            MacOS.SetEventHandler()
            MacOS.SchedParams(*self.schedparams)
        if onoff:
            MacOS.SetEventHandler(self.dispatch)
            doint, dummymask, benice, howoften, bgyield = \
                   self.schedparams
            MacOS.SchedParams(doint, everyEvent, benice,
                      howoften, bgyield)
        self._doing_asyncevents = onoff
        return old

    def do_dialogevent(self, event):
        gotone, dlg, item = DialogSelect(event)
        if gotone:
            window = dlg.GetDialogWindow()
            if window in self._windows:
                self._windows[window].do_itemhit(item, event)
            else:
                print 'Dialog event for unknown dialog'
            return 1
        return 0

    def do_mouseDown(self, event):
        (what, message, when, where, modifiers) = event
        partcode, wid = FindWindow(where)

        #
        # Find the correct name.
        #
        if partcode in partname:
            name = "do_" + partname[partcode]
        else:
            name = "do_%d" % partcode

        if wid is None:
            # No window, or a non-python window
            try:
                handler = getattr(self, name)
            except AttributeError:
                # Not menubar or something, so assume someone
                # else's window
                if hasattr(MacOS, 'HandleEvent'):
                    MacOS.HandleEvent(event)
                return
        elif wid in self._windows:
            # It is a window. Hand off to correct window.
            window = self._windows[wid]
            try:
                handler = getattr(window, name)
            except AttributeError:
                handler = self.do_unknownpartcode
        else:
            # It is a python-toolbox window, but not ours.
            handler = self.do_unknownwindow
        handler(partcode, wid, event)

    def do_inSysWindow(self, partcode, window, event):
        if hasattr(MacOS, 'HandleEvent'):
            MacOS.HandleEvent(event)

    def do_inDesk(self, partcode, window, event):
        if hasattr(MacOS, 'HandleEvent'):
            MacOS.HandleEvent(event)

    def do_inMenuBar(self, partcode, window, event):
        if not self.menubar:
            if hasattr(MacOS, 'HandleEvent'):
                MacOS.HandleEvent(event)
            return
        (what, message, when, where, modifiers) = event
        result = MenuSelect(where)
        id = (result>>16) & 0xffff      # Hi word
        if id >= 0x8000:
            id = -65536 + id
        item = result & 0xffff      # Lo word
        self.do_rawmenu(id, item, window, event)

    def do_rawmenu(self, id, item, window, event):
        try:
            self.do_menu(id, item, window, event)
        finally:
            HiliteMenu(0)

    def do_menu(self, id, item, window, event):
        if hasattr(MacOS, 'OutputSeen'):
            MacOS.OutputSeen()
        self.menubar.dispatch(id, item, window, event)


    def do_unknownpartcode(self, partcode, window, event):
        (what, message, when, where, modifiers) = event
        if DEBUG: print "Mouse down at global:", where
        if DEBUG: print "\tUnknown part code:", partcode
        if DEBUG: print "\tEvent:", self.printevent(event)
        if hasattr(MacOS, 'HandleEvent'):
            MacOS.HandleEvent(event)

    def do_unknownwindow(self, partcode, window, event):
        if DEBUG: print 'Unknown window:', window
        if hasattr(MacOS, 'HandleEvent'):
            MacOS.HandleEvent(event)

    def do_keyDown(self, event):
        self.do_key(event)

    def do_autoKey(self, event):
        if not event[-1] & cmdKey:
            self.do_key(event)

    def do_key(self, event):
        (what, message, when, where, modifiers) = event
        c = chr(message & charCodeMask)
        if self.menubar:
            result = MenuEvent(event)
            id = (result>>16) & 0xffff      # Hi word
            item = result & 0xffff      # Lo word
            if id:
                self.do_rawmenu(id, item, None, event)
                return
            # Otherwise we fall-through
        if modifiers & cmdKey:
            if c == '.':
                raise self
            else:
                if not self.menubar:
                    if hasattr(MacOS, 'HandleEvent'):
                        MacOS.HandleEvent(event)
                return
        else:
            # See whether the front window wants it
            w = MyFrontWindow()
            if w and w in self._windows:
                window = self._windows[w]
                try:
                    do_char = window.do_char
                except AttributeError:
                    do_char = self.do_char
                do_char(c, event)
            # else it wasn't for us, sigh...

    def do_char(self, c, event):
        if DEBUG: print "Character", repr(c)

    def do_updateEvt(self, event):
        (what, message, when, where, modifiers) = event
        wid = WhichWindow(message)
        if wid and wid in self._windows:
            window = self._windows[wid]
            window.do_rawupdate(wid, event)
        else:
            if hasattr(MacOS, 'HandleEvent'):
                MacOS.HandleEvent(event)

    def do_activateEvt(self, event):
        (what, message, when, where, modifiers) = event
        wid = WhichWindow(message)
        if wid and wid in self._windows:
            window = self._windows[wid]
            window.do_activate(modifiers & 1, event)
        else:
            if hasattr(MacOS, 'HandleEvent'):
                MacOS.HandleEvent(event)

    def do_osEvt(self, event):
        (what, message, when, where, modifiers) = event
        which = (message >> 24) & 0xff
        if which == 1:  # suspend/resume
            self.do_suspendresume(event)
        else:
            if DEBUG:
                print 'unknown osEvt:',
                self.printevent(event)

    def do_suspendresume(self, event):
        (what, message, when, where, modifiers) = event
        wid = MyFrontWindow()
        if wid and wid in self._windows:
            window = self._windows[wid]
            window.do_activate(message & 1, event)

    def do_kHighLevelEvent(self, event):
        (what, message, when, where, modifiers) = event
        if DEBUG:
            print "High Level Event:",
            self.printevent(event)
        try:
            AEProcessAppleEvent(event)
        except:
            pass
            #print "AEProcessAppleEvent error:"
            #traceback.print_exc()

    def do_unknownevent(self, event):
        if DEBUG:
            print "Unhandled event:",
            self.printevent(event)

    def printevent(self, event):
        (what, message, when, where, modifiers) = event
        nicewhat = repr(what)
        if what in eventname:
            nicewhat = eventname[what]
        print nicewhat,
        if what == kHighLevelEvent:
            h, v = where
            print repr(ostypecode(message)), hex(when), repr(ostypecode(h | (v<<16))),
        else:
            print hex(message), hex(when), where,
        print hex(modifiers)


class MenuBar:
    """Represent a set of menus in a menu bar.

    Interface:

    - (constructor)
    - (destructor)
    - addmenu
    - addpopup (normally used internally)
    - dispatch (called from Application)
    """

    nextid = 1      # Necessarily a class variable

    def getnextid(self):
        id = MenuBar.nextid
        MenuBar.nextid = id+1
        return id

    def __init__(self, parent=None):
        self.parent = parent
        ClearMenuBar()
        self.bar = GetMenuBar()
        self.menus = {}

    # XXX necessary?
    def close(self):
        self.parent = None
        self.bar = None
        self.menus = None

    def addmenu(self, title, after = 0, id=None):
        if id is None:
            id = self.getnextid()
        if DEBUG: print 'Newmenu', title, id # XXXX
        m = NewMenu(id, title)
        m.InsertMenu(after)
        if after >= 0:
            if self.parent:
                self.parent.needmenubarredraw = 1
            else:
                DrawMenuBar()
        return id, m

    def delmenu(self, id):
        if DEBUG: print 'Delmenu', id # XXXX
        DeleteMenu(id)

    def addpopup(self, title = ''):
        return self.addmenu(title, -1)

# Useless:
#       def install(self):
#           if not self.bar: return
#           SetMenuBar(self.bar)
#           if self.parent:
#               self.parent.needmenubarredraw = 1
#           else:
#               DrawMenuBar()

    def fixmenudimstate(self):
        for m in self.menus.keys():
            menu = self.menus[m]
            if menu.__class__ == FrameWork.AppleMenu:
                continue
            for i in range(len(menu.items)):
                label, shortcut, callback, kind = menu.items[i]
                if type(callback) == types.StringType:
                    wid = MyFrontWindow()
                    if wid and wid in self.parent._windows:
                        window = self.parent._windows[wid]
                        if hasattr(window, "domenu_" + callback):
                            menu.menu.EnableMenuItem(i + 1)
                        elif hasattr(self.parent, "domenu_" + callback):
                            menu.menu.EnableMenuItem(i + 1)
                        else:
                            menu.menu.DisableMenuItem(i + 1)
                    elif hasattr(self.parent, "domenu_" + callback):
                        menu.menu.EnableMenuItem(i + 1)
                    else:
                        menu.menu.DisableMenuItem(i + 1)
                elif callback:
                    pass

    def dispatch(self, id, item, window, event):
        if id in self.menus:
            self.menus[id].dispatch(id, item, window, event)
        else:
            if DEBUG: print "MenuBar.dispatch(%d, %d, %s, %s)" % \
                (id, item, window, event)


# XXX Need a way to get menus as resources and bind them to callbacks

class Menu:
    "One menu."

    def __init__(self, bar, title, after=0, id=None):
        self.bar = bar
        self.id, self.menu = self.bar.addmenu(title, after, id)
        bar.menus[self.id] = self
        self.items = []
        self._parent = None

    def delete(self):
        self.bar.delmenu(self.id)
        del self.bar.menus[self.id]
        self.menu.DisposeMenu()
        del self.bar
        del self.items
        del self.menu
        del self.id
        del self._parent

    def additem(self, label, shortcut=None, callback=None, kind=None):
        self.menu.AppendMenu('x')           # add a dummy string
        self.items.append((label, shortcut, callback, kind))
        item = len(self.items)
        if isinstance(label, unicode):
            self.menu.SetMenuItemTextWithCFString(item, label)
        else:
            self.menu.SetMenuItemText(item, label)
        if shortcut and type(shortcut) == type(()):
            modifiers, char = shortcut[:2]
            self.menu.SetItemCmd(item, ord(char))
            self.menu.SetMenuItemModifiers(item, modifiers)
            if len(shortcut) > 2:
                self.menu.SetMenuItemKeyGlyph(item, shortcut[2])
        elif shortcut:
            self.menu.SetItemCmd(item, ord(shortcut))
        return item

    def delitem(self, item):
        if item != len(self.items):
            raise 'Can only delete last item of a menu'
        self.menu.DeleteMenuItem(item)
        del self.items[item-1]

    def addcheck(self, label, shortcut=None, callback=None):
        return self.additem(label, shortcut, callback, 'check')

    def addradio(self, label, shortcut=None, callback=None):
        return self.additem(label, shortcut, callback, 'radio')

    def addseparator(self):
        self.menu.AppendMenu('(-')
        self.items.append(('', None, None, 'separator'))

    def addsubmenu(self, label, title=''):
        sub = Menu(self.bar, title, -1)
        item = self.additem(label, '\x1B', None, 'submenu')
        self.menu.SetItemMark(item, sub.id)
        sub._parent = self
        sub._parent_item = item
        return sub

    def dispatch(self, id, item, window, event):
        title, shortcut, callback, mtype = self.items[item-1]
        if callback:
            if not self.bar.parent or type(callback) != types.StringType:
                menuhandler = callback
            else:
                # callback is string
                wid = MyFrontWindow()
                if wid and wid in self.bar.parent._windows:
                    window = self.bar.parent._windows[wid]
                    if hasattr(window, "domenu_" + callback):
                        menuhandler = getattr(window, "domenu_" + callback)
                    elif hasattr(self.bar.parent, "domenu_" + callback):
                        menuhandler = getattr(self.bar.parent, "domenu_" + callback)
                    else:
                        # nothing we can do. we shouldn't have come this far
                        # since the menu item should have been disabled...
                        return
                elif hasattr(self.bar.parent, "domenu_" + callback):
                    menuhandler = getattr(self.bar.parent, "domenu_" + callback)
                else:
                    # nothing we can do. we shouldn't have come this far
                    # since the menu item should have been disabled...
                    return
            menuhandler(id, item, window, event)

    def enable(self, onoff):
        if onoff:
            self.menu.EnableMenuItem(0)
            if self._parent:
                self._parent.menu.EnableMenuItem(self._parent_item)
        else:
            self.menu.DisableMenuItem(0)
            if self._parent:
                self._parent.menu.DisableMenuItem(self._parent_item)
        if self.bar and self.bar.parent:
            self.bar.parent.needmenubarredraw = 1

class PopupMenu(Menu):
    def __init__(self, bar):
        Menu.__init__(self, bar, '(popup)', -1)

    def popup(self, x, y, event, default=1, window=None):
        # NOTE that x and y are global coordinates, and they should probably
        # be topleft of the button the user clicked (not mouse-coordinates),
        # so the popup nicely overlaps.
        reply = self.menu.PopUpMenuSelect(x, y, default)
        if not reply:
            return
        id = (reply >> 16) & 0xffff
        item = reply & 0xffff
        if not window:
            wid = MyFrontWindow()
            try:
                window = self.bar.parent._windows[wid]
            except:
                pass # If we can't find the window we pass None
        self.dispatch(id, item, window, event)

class MenuItem:
    def __init__(self, menu, title, shortcut=None, callback=None, kind=None):
        self.item = menu.additem(title, shortcut, callback)
        self.menu = menu

    def delete(self):
        self.menu.delitem(self.item)
        del self.menu
        del self.item

    def check(self, onoff):
        self.menu.menu.CheckMenuItem(self.item, onoff)

    def enable(self, onoff):
        if onoff:
            self.menu.menu.EnableMenuItem(self.item)
        else:
            self.menu.menu.DisableMenuItem(self.item)

    def settext(self, text):
        self.menu.menu.SetMenuItemText(self.item, text)

    def setstyle(self, style):
        self.menu.menu.SetItemStyle(self.item, style)

    def seticon(self, icon):
        self.menu.menu.SetItemIcon(self.item, icon)

    def setcmd(self, cmd):
        self.menu.menu.SetItemCmd(self.item, cmd)

    def setmark(self, cmd):
        self.menu.menu.SetItemMark(self.item, cmd)


class RadioItem(MenuItem):
    def __init__(self, menu, title, shortcut=None, callback=None):
        MenuItem.__init__(self, menu, title, shortcut, callback, 'radio')

class CheckItem(MenuItem):
    def __init__(self, menu, title, shortcut=None, callback=None):
        MenuItem.__init__(self, menu, title, shortcut, callback, 'check')

def Separator(menu):
    menu.addseparator()

def SubMenu(menu, label, title=''):
    return menu.addsubmenu(label, title)


class AppleMenu(Menu):

    def __init__(self, bar, abouttext="About me...", aboutcallback=None):
        Menu.__init__(self, bar, "\024", id=SIOUX_APPLEMENU_ID)
        if MacOS.runtimemodel == 'ppc':
            self.additem(abouttext, None, aboutcallback)
            self.addseparator()
            self.menu.AppendResMenu('DRVR')
        else:
            # Additem()'s tricks do not work for "apple" menu under Carbon
            self.menu.InsertMenuItem(abouttext, 0)
            self.items.append((abouttext, None, aboutcallback, None))

    def dispatch(self, id, item, window, event):
        if item == 1:
            Menu.dispatch(self, id, item, window, event)
        elif MacOS.runtimemodel == 'ppc':
            name = self.menu.GetMenuItemText(item)
            OpenDeskAcc(name)

class HelpMenu(Menu):
    def __init__(self, bar):
        # Note we don't call Menu.__init__, we do the necessary things by hand
        self.bar = bar
        self.menu, index = HMGetHelpMenu()
        self.id = self.menu.GetMenuID()
        bar.menus[self.id] = self
        # The next line caters for the entries the system already handles for us
        self.items = [None]*(index-1)
        self._parent = None


class Window:
    """A single window belonging to an application"""

    def __init__(self, parent):
        self.wid = None
        self.parent = parent

    def open(self, bounds=(40, 40, 400, 400), resid=None):
        if resid != None:
            self.wid = GetNewWindow(resid, -1)
        else:
            self.wid = NewWindow(bounds, self.__class__.__name__, 1,
                8, -1, 1, 0)    # changed to proc id 8 to include zoom box. jvr
        self.do_postopen()

    def do_postopen(self):
        """Tell our parent we exist"""
        self.parent.appendwindow(self.wid, self)

    def close(self):
        self.do_postclose()

    def do_postclose(self):
        self.parent.removewindow(self.wid)
        self.parent = None
        self.wid = None

    def SetPort(self):
        # Convinience method
        SetPort(self.wid)

    def GetWindow(self):
        return self.wid

    def do_inDrag(self, partcode, window, event):
        where = event[3]
        window.DragWindow(where, self.draglimit)

    draglimit = screenbounds

    def do_inGoAway(self, partcode, window, event):
        where = event[3]
        if window.TrackGoAway(where):
            self.close()

    def do_inZoom(self, partcode, window, event):
        (what, message, when, where, modifiers) = event
        if window.TrackBox(where, partcode):
            window.ZoomWindow(partcode, 1)
            rect = window.GetWindowUserState()                  # so that zoom really works... jvr
            self.do_postresize(rect[2] - rect[0], rect[3] - rect[1], window)    # jvr

    def do_inZoomIn(self, partcode, window, event):
        SetPort(window) # !!!
        self.do_inZoom(partcode, window, event)

    def do_inZoomOut(self, partcode, window, event):
        SetPort(window) # !!!
        self.do_inZoom(partcode, window, event)

    def do_inGrow(self, partcode, window, event):
        (what, message, when, where, modifiers) = event
        result = window.GrowWindow(where, self.growlimit)
        if result:
            height = (result>>16) & 0xffff  # Hi word
            width = result & 0xffff     # Lo word
            self.do_resize(width, height, window)

    growlimit = (50, 50, screenbounds[2] - screenbounds[0], screenbounds[3] - screenbounds[1])      # jvr

    def do_resize(self, width, height, window):
        l, t, r, b = self.wid.GetWindowPort().GetPortBounds()           # jvr, forGrowIcon
        self.SetPort()                          # jvr
        self.wid.InvalWindowRect((r - SCROLLBARWIDTH + 1, b - SCROLLBARWIDTH + 1, r, b))    # jvr
        window.SizeWindow(width, height, 1)         # changed updateFlag to true jvr
        self.do_postresize(width, height, window)

    def do_postresize(self, width, height, window):
        SetPort(window)
        self.wid.InvalWindowRect(window.GetWindowPort().GetPortBounds())

    def do_inContent(self, partcode, window, event):
        #
        # If we're not frontmost, select ourselves and wait for
        # the activate event.
        #
        if MyFrontWindow() != window:
            window.SelectWindow()
            return
        # We are. Handle the event.
        (what, message, when, where, modifiers) = event
        SetPort(window)
        local = GlobalToLocal(where)
        self.do_contentclick(local, modifiers, event)

    def do_contentclick(self, local, modifiers, event):
        if DEBUG:
            print 'Click in contents at %s, modifiers %s'%(local, modifiers)

    def do_rawupdate(self, window, event):
        if DEBUG: print "raw update for", window
        SetPort(window)
        window.BeginUpdate()
        self.do_update(window, event)
        window.EndUpdate()

    def do_update(self, window, event):
        if DEBUG:
            import time
            for i in range(8):
                time.sleep(0.1)
                InvertRgn(window.GetWindowPort().visRgn)
            FillRgn(window.GetWindowPort().visRgn, GetQDGlobalsGray())
        else:
            EraseRgn(window.GetWindowPort().visRgn)

    def do_activate(self, activate, event):
        if DEBUG: print 'Activate %d for %s'%(activate, self.wid)

class ControlsWindow(Window):

    def do_rawupdate(self, window, event):
        if DEBUG: print "raw update for", window
        SetPort(window)
        window.BeginUpdate()
        self.do_update(window, event)
        #DrawControls(window)                   # jvr
        UpdateControls(window, window.GetWindowPort().visRgn)   # jvr
        window.DrawGrowIcon()
        window.EndUpdate()

    def do_controlhit(self, window, control, pcode, event):
        if DEBUG: print "control hit in", window, "on", control, "; pcode =", pcode

    def do_inContent(self, partcode, window, event):
        if MyFrontWindow() != window:
            window.SelectWindow()
            return
        (what, message, when, where, modifiers) = event
        SetPort(window)  # XXXX Needed?
        local = GlobalToLocal(where)
        pcode, control = FindControl(local, window)
        if pcode and control:
            self.do_rawcontrolhit(window, control, pcode, local, event)
        else:
            if DEBUG: print "FindControl(%s, %s) -> (%s, %s)" % \
                (local, window, pcode, control)
            self.do_contentclick(local, modifiers, event)

    def do_rawcontrolhit(self, window, control, pcode, local, event):
        pcode = control.TrackControl(local)
        if pcode:
            self.do_controlhit(window, control, pcode, event)

class ScrolledWindow(ControlsWindow):
    def __init__(self, parent):
        self.barx = self.bary = None
        self.barx_enabled = self.bary_enabled = 1
        self.activated = 1
        ControlsWindow.__init__(self, parent)

    def scrollbars(self, wantx=1, wanty=1):
        SetPort(self.wid)
        self.barx = self.bary = None
        self.barx_enabled = self.bary_enabled = 1
        x0, y0, x1, y1 = self.wid.GetWindowPort().GetPortBounds()
        vx, vy = self.getscrollbarvalues()
        if vx is None: self.barx_enabled, vx = 0, 0
        if vy is None: self.bary_enabled, vy = 0, 0
        if wantx:
            rect = x0-1, y1-(SCROLLBARWIDTH-1), x1-(SCROLLBARWIDTH-2), y1+1
            self.barx = NewControl(self.wid, rect, "", 1, vx, 0, 32767, 16, 0)
            if not self.barx_enabled: self.barx.HiliteControl(255)
##              self.wid.InvalWindowRect(rect)
        if wanty:
            rect = x1-(SCROLLBARWIDTH-1), y0-1, x1+1, y1-(SCROLLBARWIDTH-2)
            self.bary = NewControl(self.wid, rect, "", 1, vy, 0, 32767, 16, 0)
            if not self.bary_enabled: self.bary.HiliteControl(255)
##              self.wid.InvalWindowRect(rect)

    def do_postclose(self):
        self.barx = self.bary = None
        ControlsWindow.do_postclose(self)

    def do_activate(self, onoff, event):
        self.activated = onoff
        if onoff:
            if self.barx and self.barx_enabled:
                self.barx.ShowControl() # jvr
            if self.bary and self.bary_enabled:
                self.bary.ShowControl() # jvr
        else:
            if self.barx:
                self.barx.HideControl() # jvr; An inactive window should have *hidden*
                            # scrollbars, not just dimmed (no matter what
                            # BBEdit does... look at the Finder)
            if self.bary:
                self.bary.HideControl() # jvr
        self.wid.DrawGrowIcon()         # jvr

    def do_postresize(self, width, height, window):
        l, t, r, b = self.wid.GetWindowPort().GetPortBounds()
        self.SetPort()
        if self.barx:
            self.barx.HideControl()     # jvr
            self.barx.MoveControl(l-1, b-(SCROLLBARWIDTH-1))
            self.barx.SizeControl((r-l)-(SCROLLBARWIDTH-3), SCROLLBARWIDTH) # jvr
        if self.bary:
            self.bary.HideControl()     # jvr
            self.bary.MoveControl(r-(SCROLLBARWIDTH-1), t-1)
            self.bary.SizeControl(SCROLLBARWIDTH, (b-t)-(SCROLLBARWIDTH-3)) # jvr
        if self.barx:
            self.barx.ShowControl()     # jvr
            self.wid.ValidWindowRect((l, b - SCROLLBARWIDTH + 1, r - SCROLLBARWIDTH + 2, b))    # jvr
        if self.bary:
            self.bary.ShowControl()     # jvr
            self.wid.ValidWindowRect((r - SCROLLBARWIDTH + 1, t, r, b - SCROLLBARWIDTH + 2))    # jvr
        self.wid.InvalWindowRect((r - SCROLLBARWIDTH + 1, b - SCROLLBARWIDTH + 1, r, b))    # jvr, growicon


    def do_rawcontrolhit(self, window, control, pcode, local, event):
        if control == self.barx:
            which = 'x'
        elif control == self.bary:
            which = 'y'
        else:
            return 0
        if pcode in (inUpButton, inDownButton, inPageUp, inPageDown):
            # We do the work for the buttons and grey area in the tracker
            dummy = control.TrackControl(local, self.do_controltrack)
        else:
            # but the thumb is handled here
            pcode = control.TrackControl(local)
            if pcode == inThumb:
                value = control.GetControlValue()
                print 'setbars', which, value #DBG
                self.scrollbar_callback(which, 'set', value)
                self.updatescrollbars()
            else:
                print 'funny part', pcode #DBG
        return 1

    def do_controltrack(self, control, pcode):
        if control == self.barx:
            which = 'x'
        elif control == self.bary:
            which = 'y'
        else:
            return

        if pcode == inUpButton:
            what = '-'
        elif pcode == inDownButton:
            what = '+'
        elif pcode == inPageUp:
            what = '--'
        elif pcode == inPageDown:
            what = '++'
        else:
            return
        self.scrollbar_callback(which, what, None)
        self.updatescrollbars()

    def updatescrollbars(self):
        SetPort(self.wid)
        vx, vy = self.getscrollbarvalues()
        if self.barx:
            if vx is None:
                self.barx.HiliteControl(255)
                self.barx_enabled = 0
            else:
                if not self.barx_enabled:
                    self.barx_enabled = 1
                    if self.activated:
                        self.barx.HiliteControl(0)
                self.barx.SetControlValue(vx)
        if self.bary:
            if vy is None:
                self.bary.HiliteControl(255)
                self.bary_enabled = 0
            else:
                if not self.bary_enabled:
                    self.bary_enabled = 1
                    if self.activated:
                        self.bary.HiliteControl(0)
                self.bary.SetControlValue(vy)

    # Auxiliary function: convert standard text/image/etc coordinate
    # to something palatable as getscrollbarvalues() return
    def scalebarvalue(self, absmin, absmax, curmin, curmax):
        if curmin <= absmin and curmax >= absmax:
            return None
        if curmin <= absmin:
            return 0
        if curmax >= absmax:
            return 32767
        perc = float(curmin-absmin)/float(absmax-absmin)
        return int(perc*32767)

    # To be overridden:

    def getscrollbarvalues(self):
        return 0, 0

    def scrollbar_callback(self, which, what, value):
        print 'scroll', which, what, value

class DialogWindow(Window):
    """A modeless dialog window"""

    def open(self, resid):
        self.dlg = GetNewDialog(resid, -1)
        self.wid = self.dlg.GetDialogWindow()
        self.do_postopen()

    def close(self):
        self.do_postclose()

    def do_postclose(self):
        self.dlg = None
        Window.do_postclose(self)

    def do_itemhit(self, item, event):
        print 'Dialog %s, item %d hit'%(self.dlg, item)

    def do_rawupdate(self, window, event):
        pass

def ostypecode(x):
    "Convert a long int to the 4-character code it really is"
    s = ''
    for i in range(4):
        x, c = divmod(x, 256)
        s = chr(c) + s
    return s


class TestApp(Application):

    "This class is used by the test() function"

    def makeusermenus(self):
        self.filemenu = m = Menu(self.menubar, "File")
        self.saveitem = MenuItem(m, "Save", "S", self.save)
        Separator(m)
        self.optionsmenu = mm = SubMenu(m, "Options")
        self.opt1 = CheckItem(mm, "Arguments", "A")
        self.opt2 = CheckItem(mm, "Being hit on the head lessons", (kMenuOptionModifier, "A"))
        self.opt3 = CheckItem(mm, "Complaints", (kMenuOptionModifier|kMenuNoCommandModifier, "A"))
        Separator(m)
        self.itemeh = MenuItem(m, "Enable Help", None, self.enablehelp)
        self.itemdbg = MenuItem(m, "Debug", None, self.debug)
        Separator(m)
        self.quititem = MenuItem(m, "Quit", "Q", self.quit)

    def save(self, *args):
        print "Save"

    def quit(self, *args):
        raise self

    def enablehelp(self, *args):
        hm = self.gethelpmenu()
        self.nohelpitem = MenuItem(hm, "There isn't any", None, self.nohelp)

    def nohelp(self, *args):
        print "I told you there isn't any!"

    def debug(self, *args):
        import pdb
        pdb.set_trace()


def test():
    "Test program"
    app = TestApp()
    app.mainloop()


if __name__ == '__main__':
    test()



from test.test_support import run_unittest
import unittest
import sys
import imp
import pkgutil
import os
import os.path
import tempfile
import shutil
import zipfile



class PkgutilTests(unittest.TestCase):

    def setUp(self):
        self.dirname = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, self.dirname)
        sys.path.insert(0, self.dirname)

    def tearDown(self):
        del sys.path[0]

    def test_getdata_filesys(self):
        pkg = 'test_getdata_filesys'

        # Include a LF and a CRLF, to test that binary data is read back
        RESOURCE_DATA = 'Hello, world!\nSecond line\r\nThird line'

        # Make a package with some resources
        package_dir = os.path.join(self.dirname, pkg)
        os.mkdir(package_dir)
        # Empty init.py
        f = open(os.path.join(package_dir, '__init__.py'), "wb")
        f.close()
        # Resource files, res.txt, sub/res.txt
        f = open(os.path.join(package_dir, 'res.txt'), "wb")
        f.write(RESOURCE_DATA)
        f.close()
        os.mkdir(os.path.join(package_dir, 'sub'))
        f = open(os.path.join(package_dir, 'sub', 'res.txt'), "wb")
        f.write(RESOURCE_DATA)
        f.close()

        # Check we can read the resources
        res1 = pkgutil.get_data(pkg, 'res.txt')
        self.assertEqual(res1, RESOURCE_DATA)
        res2 = pkgutil.get_data(pkg, 'sub/res.txt')
        self.assertEqual(res2, RESOURCE_DATA)

        del sys.modules[pkg]

    def test_getdata_zipfile(self):
        zip = 'test_getdata_zipfile.zip'
        pkg = 'test_getdata_zipfile'

        # Include a LF and a CRLF, to test that binary data is read back
        RESOURCE_DATA = 'Hello, world!\nSecond line\r\nThird line'

        # Make a package with some resources
        zip_file = os.path.join(self.dirname, zip)
        z = zipfile.ZipFile(zip_file, 'w')

        # Empty init.py
        z.writestr(pkg + '/__init__.py', "")
        # Resource files, res.txt, sub/res.txt
        z.writestr(pkg + '/res.txt', RESOURCE_DATA)
        z.writestr(pkg + '/sub/res.txt', RESOURCE_DATA)
        z.close()

        # Check we can read the resources
        sys.path.insert(0, zip_file)
        res1 = pkgutil.get_data(pkg, 'res.txt')
        self.assertEqual(res1, RESOURCE_DATA)
        res2 = pkgutil.get_data(pkg, 'sub/res.txt')
        self.assertEqual(res2, RESOURCE_DATA)
        del sys.path[0]

        del sys.modules[pkg]

    def test_unreadable_dir_on_syspath(self):
        # issue7367 - walk_packages failed if unreadable dir on sys.path
        package_name = "unreadable_package"
        d = os.path.join(self.dirname, package_name)
        # this does not appear to create an unreadable dir on Windows
        #   but the test should not fail anyway
        os.mkdir(d, 0)
        self.addCleanup(os.rmdir, d)
        for t in pkgutil.walk_packages(path=[self.dirname]):
            self.fail("unexpected package found")

class PkgutilPEP302Tests(unittest.TestCase):

    class MyTestLoader(object):
        def load_module(self, fullname):
            # Create an empty module
            mod = sys.modules.setdefault(fullname, imp.new_module(fullname))
            mod.__file__ = "<%s>" % self.__class__.__name__
            mod.__loader__ = self
            # Make it a package
            mod.__path__ = []
            # Count how many times the module is reloaded
            mod.__dict__['loads'] = mod.__dict__.get('loads',0) + 1
            return mod

        def get_data(self, path):
            return "Hello, world!"

    class MyTestImporter(object):
        def find_module(self, fullname, path=None):
            return PkgutilPEP302Tests.MyTestLoader()

    def setUp(self):
        sys.meta_path.insert(0, self.MyTestImporter())

    def tearDown(self):
        del sys.meta_path[0]

    def test_getdata_pep302(self):
        # Use a dummy importer/loader
        self.assertEqual(pkgutil.get_data('foo', 'dummy'), "Hello, world!")
        del sys.modules['foo']

    def test_alreadyloaded(self):
        # Ensure that get_data works without reloading - the "loads" module
        # variable in the example loader should count how many times a reload
        # occurs.
        import foo
        self.assertEqual(foo.loads, 1)
        self.assertEqual(pkgutil.get_data('foo', 'dummy'), "Hello, world!")
        self.assertEqual(foo.loads, 1)
        del sys.modules['foo']

def test_main():
    run_unittest(PkgutilTests, PkgutilPEP302Tests)
    # this is necessary if test is run repeated (like when finding leaks)
    import zipimport
    zipimport._zip_directory_cache.clear()

if __name__ == '__main__':
    test_main()



"""
Basic TestCases for BTree and hash DBs, with and without a DBEnv, with
various DB flags, etc.
"""

import os
import errno
import string
from pprint import pprint
import unittest
import time
import sys

from test_all import db, test_support, verbose, get_new_environment_path, \
        get_new_database_path

DASH = '-'


#----------------------------------------------------------------------

class VersionTestCase(unittest.TestCase):
    def test00_version(self):
        info = db.version()
        if verbose:
            print '\n', '-=' * 20
            print 'bsddb.db.version(): %s' % (info, )
            print db.DB_VERSION_STRING
            print '-=' * 20
        self.assertEqual(info, (db.DB_VERSION_MAJOR, db.DB_VERSION_MINOR,
                        db.DB_VERSION_PATCH))

#----------------------------------------------------------------------

class BasicTestCase(unittest.TestCase):
    dbtype       = db.DB_UNKNOWN  # must be set in derived class
    cachesize    = (0, 1024*1024, 1)
    dbopenflags  = 0
    dbsetflags   = 0
    dbmode       = 0660
    dbname       = None
    useEnv       = 0
    envflags     = 0
    envsetflags  = 0

    _numKeys      = 1002    # PRIVATE.  NOTE: must be an even value

    def setUp(self):
        if self.useEnv:
            self.homeDir=get_new_environment_path()
            try:
                self.env = db.DBEnv()
                self.env.set_lg_max(1024*1024)
                self.env.set_tx_max(30)
                self._t = int(time.time())
                self.env.set_tx_timestamp(self._t)
                self.env.set_flags(self.envsetflags, 1)
                self.env.open(self.homeDir, self.envflags | db.DB_CREATE)
                self.filename = "test"
            # Yes, a bare except is intended, since we're re-raising the exc.
            except:
                test_support.rmtree(self.homeDir)
                raise
        else:
            self.env = None
            self.filename = get_new_database_path()

        # create and open the DB
        self.d = db.DB(self.env)
        if not self.useEnv :
            self.d.set_cachesize(*self.cachesize)
            cachesize = self.d.get_cachesize()
            self.assertEqual(cachesize[0], self.cachesize[0])
            self.assertEqual(cachesize[2], self.cachesize[2])
            # Berkeley DB expands the cache 25% accounting overhead,
            # if the cache is small.
            self.assertEqual(125, int(100.0*cachesize[1]/self.cachesize[1]))
        self.d.set_flags(self.dbsetflags)
        if self.dbname:
            self.d.open(self.filename, self.dbname, self.dbtype,
                        self.dbopenflags|db.DB_CREATE, self.dbmode)
        else:
            self.d.open(self.filename,   # try out keyword args
                        mode = self.dbmode,
                        dbtype = self.dbtype,
                        flags = self.dbopenflags|db.DB_CREATE)

        if not self.useEnv:
            self.assertRaises(db.DBInvalidArgError,
                    self.d.set_cachesize, *self.cachesize)

        self.populateDB()


    def tearDown(self):
        self.d.close()
        if self.env is not None:
            self.env.close()
            test_support.rmtree(self.homeDir)
        else:
            os.remove(self.filename)



    def populateDB(self, _txn=None):
        d = self.d

        for x in range(self._numKeys//2):
            key = '%04d' % (self._numKeys - x)  # insert keys in reverse order
            data = self.makeData(key)
            d.put(key, data, _txn)

        d.put('empty value', '', _txn)

        for x in range(self._numKeys//2-1):
            key = '%04d' % x  # and now some in forward order
            data = self.makeData(key)
            d.put(key, data, _txn)

        if _txn:
            _txn.commit()

        num = len(d)
        if verbose:
            print "created %d records" % num


    def makeData(self, key):
        return DASH.join([key] * 5)



    #----------------------------------------

    def test01_GetsAndPuts(self):
        d = self.d

        if verbose:
            print '\n', '-=' * 30
            print "Running %s.test01_GetsAndPuts..." % self.__class__.__name__

        for key in ['0001', '0100', '0400', '0700', '0999']:
            data = d.get(key)
            if verbose:
                print data

        self.assertEqual(d.get('0321'), '0321-0321-0321-0321-0321')

        # By default non-existent keys return None...
        self.assertEqual(d.get('abcd'), None)

        # ...but they raise exceptions in other situations.  Call
        # set_get_returns_none() to change it.
        try:
            d.delete('abcd')
        except db.DBNotFoundError, val:
            if sys.version_info < (2, 6) :
                self.assertEqual(val[0], db.DB_NOTFOUND)
            else :
                self.assertEqual(val.args[0], db.DB_NOTFOUND)
            if verbose: print val
        else:
            self.fail("expected exception")


        d.put('abcd', 'a new record')
        self.assertEqual(d.get('abcd'), 'a new record')

        d.put('abcd', 'same key')
        if self.dbsetflags & db.DB_DUP:
            self.assertEqual(d.get('abcd'), 'a new record')
        else:
            self.assertEqual(d.get('abcd'), 'same key')


        try:
            d.put('abcd', 'this should fail', flags=db.DB_NOOVERWRITE)
        except db.DBKeyExistError, val:
            if sys.version_info < (2, 6) :
                self.assertEqual(val[0], db.DB_KEYEXIST)
            else :
                self.assertEqual(val.args[0], db.DB_KEYEXIST)
            if verbose: print val
        else:
            self.fail("expected exception")

        if self.dbsetflags & db.DB_DUP:
            self.assertEqual(d.get('abcd'), 'a new record')
        else:
            self.assertEqual(d.get('abcd'), 'same key')


        d.sync()
        d.close()
        del d

        self.d = db.DB(self.env)
        if self.dbname:
            self.d.open(self.filename, self.dbname)
        else:
            self.d.open(self.filename)
        d = self.d

        self.assertEqual(d.get('0321'), '0321-0321-0321-0321-0321')
        if self.dbsetflags & db.DB_DUP:
            self.assertEqual(d.get('abcd'), 'a new record')
        else:
            self.assertEqual(d.get('abcd'), 'same key')

        rec = d.get_both('0555', '0555-0555-0555-0555-0555')
        if verbose:
            print rec

        self.assertEqual(d.get_both('0555', 'bad data'), None)

        # test default value
        data = d.get('bad key', 'bad data')
        self.assertEqual(data, 'bad data')

        # any object can pass through
        data = d.get('bad key', self)
        self.assertEqual(data, self)

        s = d.stat()
        self.assertEqual(type(s), type({}))
        if verbose:
            print 'd.stat() returned this dictionary:'
            pprint(s)


    #----------------------------------------

    def test02_DictionaryMethods(self):
        d = self.d

        if verbose:
            print '\n', '-=' * 30
            print "Running %s.test02_DictionaryMethods..." % \
                  self.__class__.__name__

        for key in ['0002', '0101', '0401', '0701', '0998']:
            data = d[key]
            self.assertEqual(data, self.makeData(key))
            if verbose:
                print data

        self.assertEqual(len(d), self._numKeys)
        keys = d.keys()
        self.assertEqual(len(keys), self._numKeys)
        self.assertEqual(type(keys), type([]))

        d['new record'] = 'a new record'
        self.assertEqual(len(d), self._numKeys+1)
        keys = d.keys()
        self.assertEqual(len(keys), self._numKeys+1)

        d['new record'] = 'a replacement record'
        self.assertEqual(len(d), self._numKeys+1)
        keys = d.keys()
        self.assertEqual(len(keys), self._numKeys+1)

        if verbose:
            print "the first 10 keys are:"
            pprint(keys[:10])

        self.assertEqual(d['new record'], 'a replacement record')

# We check also the positional parameter
        self.assertEqual(d.has_key('0001', None), 1)
# We check also the keyword parameter
        self.assertEqual(d.has_key('spam', txn=None), 0)

        items = d.items()
        self.assertEqual(len(items), self._numKeys+1)
        self.assertEqual(type(items), type([]))
        self.assertEqual(type(items[0]), type(()))
        self.assertEqual(len(items[0]), 2)

        if verbose:
            print "the first 10 items are:"
            pprint(items[:10])

        values = d.values()
        self.assertEqual(len(values), self._numKeys+1)
        self.assertEqual(type(values), type([]))

        if verbose:
            print "the first 10 values are:"
            pprint(values[:10])


    #----------------------------------------

    def test02b_SequenceMethods(self):
        d = self.d

        for key in ['0002', '0101', '0401', '0701', '0998']:
            data = d[key]
            self.assertEqual(data, self.makeData(key))
            if verbose:
                print data

        self.assertTrue(hasattr(d, "__contains__"))
        self.assertTrue("0401" in d)
        self.assertFalse("1234" in d)


    #----------------------------------------

    def test03_SimpleCursorStuff(self, get_raises_error=0, set_raises_error=0):
        if verbose:
            print '\n', '-=' * 30
            print "Running %s.test03_SimpleCursorStuff (get_error %s, set_error %s)..." % \
                  (self.__class__.__name__, get_raises_error, set_raises_error)

        if self.env and self.dbopenflags & db.DB_AUTO_COMMIT:
            txn = self.env.txn_begin()
        else:
            txn = None
        c = self.d.cursor(txn=txn)

        rec = c.first()
        count = 0
        while rec is not None:
            count = count + 1
            if verbose and count % 100 == 0:
                print rec
            try:
                rec = c.next()
            except db.DBNotFoundError, val:
                if get_raises_error:
                    if sys.version_info < (2, 6) :
                        self.assertEqual(val[0], db.DB_NOTFOUND)
                    else :
                        self.assertEqual(val.args[0], db.DB_NOTFOUND)
                    if verbose: print val
                    rec = None
                else:
                    self.fail("unexpected DBNotFoundError")
            self.assertEqual(c.get_current_size(), len(c.current()[1]),
                    "%s != len(%r)" % (c.get_current_size(), c.current()[1]))

        self.assertEqual(count, self._numKeys)


        rec = c.last()
        count = 0
        while rec is not None:
            count = count + 1
            if verbose and count % 100 == 0:
                print rec
            try:
                rec = c.prev()
            except db.DBNotFoundError, val:
                if get_raises_error:
                    if sys.version_info < (2, 6) :
                        self.assertEqual(val[0], db.DB_NOTFOUND)
                    else :
                        self.assertEqual(val.args[0], db.DB_NOTFOUND)
                    if verbose: print val
                    rec = None
                else:
                    self.fail("unexpected DBNotFoundError")

        self.assertEqual(count, self._numKeys)

        rec = c.set('0505')
        rec2 = c.current()
        self.assertEqual(rec, rec2)
        self.assertEqual(rec[0], '0505')
        self.assertEqual(rec[1], self.makeData('0505'))
        self.assertEqual(c.get_current_size(), len(rec[1]))

        # make sure we get empty values properly
        rec = c.set('empty value')
        self.assertEqual(rec[1], '')
        self.assertEqual(c.get_current_size(), 0)

        try:
            n = c.set('bad key')
        except db.DBNotFoundError, val:
            if sys.version_info < (2, 6) :
                self.assertEqual(val[0], db.DB_NOTFOUND)
            else :
                self.assertEqual(val.args[0], db.DB_NOTFOUND)
            if verbose: print val
        else:
            if set_raises_error:
                self.fail("expected exception")
            if n is not None:
                self.fail("expected None: %r" % (n,))

        rec = c.get_both('0404', self.makeData('0404'))
        self.assertEqual(rec, ('0404', self.makeData('0404')))

        try:
            n = c.get_both('0404', 'bad data')
        except db.DBNotFoundError, val:
            if sys.version_info < (2, 6) :
                self.assertEqual(val[0], db.DB_NOTFOUND)
            else :
                self.assertEqual(val.args[0], db.DB_NOTFOUND)
            if verbose: print val
        else:
            if get_raises_error:
                self.fail("expected exception")
            if n is not None:
                self.fail("expected None: %r" % (n,))

        if self.d.get_type() == db.DB_BTREE:
            rec = c.set_range('011')
            if verbose:
                print "searched for '011', found: ", rec

            rec = c.set_range('011',dlen=0,doff=0)
            if verbose:
                print "searched (partial) for '011', found: ", rec
            if rec[1] != '': self.fail('expected empty data portion')

            ev = c.set_range('empty value')
            if verbose:
                print "search for 'empty value' returned", ev
            if ev[1] != '': self.fail('empty value lookup failed')

        c.set('0499')
        c.delete()
        try:
            rec = c.current()
        except db.DBKeyEmptyError, val:
            if get_raises_error:
                if sys.version_info < (2, 6) :
                    self.assertEqual(val[0], db.DB_KEYEMPTY)
                else :
                    self.assertEqual(val.args[0], db.DB_KEYEMPTY)
                if verbose: print val
            else:
                self.fail("unexpected DBKeyEmptyError")
        else:
            if get_raises_error:
                self.fail('DBKeyEmptyError exception expected')

        c.next()
        c2 = c.dup(db.DB_POSITION)
        self.assertEqual(c.current(), c2.current())

        c2.put('', 'a new value', db.DB_CURRENT)
        self.assertEqual(c.current(), c2.current())
        self.assertEqual(c.current()[1], 'a new value')

        c2.put('', 'er', db.DB_CURRENT, dlen=0, doff=5)
        self.assertEqual(c2.current()[1], 'a newer value')

        c.close()
        c2.close()
        if txn:
            txn.commit()

        # time to abuse the closed cursors and hope we don't crash
        methods_to_test = {
            'current': (),
            'delete': (),
            'dup': (db.DB_POSITION,),
            'first': (),
            'get': (0,),
            'next': (),
            'prev': (),
            'last': (),
            'put':('', 'spam', db.DB_CURRENT),
            'set': ("0505",),
        }
        for method, args in methods_to_test.items():
            try:
                if verbose:
                    print "attempting to use a closed cursor's %s method" % \
                          method
                # a bug may cause a NULL pointer dereference...
                getattr(c, method)(*args)
            except db.DBError, val:
                if sys.version_info < (2, 6) :
                    self.assertEqual(val[0], 0)
                else :
                    self.assertEqual(val.args[0], 0)
                if verbose: print val
            else:
                self.fail("no exception raised when using a buggy cursor's"
                          "%s method" % method)

        #
        # free cursor referencing a closed database, it should not barf:
        #
        oldcursor = self.d.cursor(txn=txn)
        self.d.close()

        # this would originally cause a segfault when the cursor for a
        # closed database was cleaned up.  it should not anymore.
        # SF pybsddb bug id 667343
        del oldcursor

    def test03b_SimpleCursorWithoutGetReturnsNone0(self):
        # same test but raise exceptions instead of returning None
        if verbose:
            print '\n', '-=' * 30
            print "Running %s.test03b_SimpleCursorStuffWithoutGetReturnsNone..." % \
                  self.__class__.__name__

        old = self.d.set_get_returns_none(0)
        self.assertEqual(old, 2)
        self.test03_SimpleCursorStuff(get_raises_error=1, set_raises_error=1)

    def test03b_SimpleCursorWithGetReturnsNone1(self):
        # same test but raise exceptions instead of returning None
        if verbose:
            print '\n', '-=' * 30
            print "Running %s.test03b_SimpleCursorStuffWithoutGetReturnsNone..." % \
                  self.__class__.__name__

        old = self.d.set_get_returns_none(1)
        self.test03_SimpleCursorStuff(get_raises_error=0, set_raises_error=1)


    def test03c_SimpleCursorGetReturnsNone2(self):
        # same test but raise exceptions instead of returning None
        if verbose:
            print '\n', '-=' * 30
            print "Running %s.test03c_SimpleCursorStuffWithoutSetReturnsNone..." % \
                  self.__class__.__name__

        old = self.d.set_get_returns_none(1)
        self.assertEqual(old, 2)
        old = self.d.set_get_returns_none(2)
        self.assertEqual(old, 1)
        self.test03_SimpleCursorStuff(get_raises_error=0, set_raises_error=0)

    if db.version() >= (4, 6):
        def test03d_SimpleCursorPriority(self) :
            c = self.d.cursor()
            c.set_priority(db.DB_PRIORITY_VERY_LOW)  # Positional
            self.assertEqual(db.DB_PRIORITY_VERY_LOW, c.get_priority())
            c.set_priority(priority=db.DB_PRIORITY_HIGH)  # Keyword
            self.assertEqual(db.DB_PRIORITY_HIGH, c.get_priority())
            c.close()

    #----------------------------------------

    def test04_PartialGetAndPut(self):
        d = self.d
        if verbose:
            print '\n', '-=' * 30
            print "Running %s.test04_PartialGetAndPut..." % \
                  self.__class__.__name__

        key = "partialTest"
        data = "1" * 1000 + "2" * 1000
        d.put(key, data)
        self.assertEqual(d.get(key), data)
        self.assertEqual(d.get(key, dlen=20, doff=990),
                ("1" * 10) + ("2" * 10))

        d.put("partialtest2", ("1" * 30000) + "robin" )
        self.assertEqual(d.get("partialtest2", dlen=5, doff=30000), "robin")

        # There seems to be a bug in DB here...  Commented out the test for
        # now.
        ##self.assertEqual(d.get("partialtest2", dlen=5, doff=30010), "")

        if self.dbsetflags != db.DB_DUP:
            # Partial put with duplicate records requires a cursor
            d.put(key, "0000", dlen=2000, doff=0)
            self.assertEqual(d.get(key), "0000")

            d.put(key, "1111", dlen=1, doff=2)
            self.assertEqual(d.get(key), "0011110")

    #----------------------------------------

    def test05_GetSize(self):
        d = self.d
        if verbose:
            print '\n', '-=' * 30
            print "Running %s.test05_GetSize..." % self.__class__.__name__

        for i in range(1, 50000, 500):
            key = "size%s" % i
            #print "before ", i,
            d.put(key, "1" * i)
            #print "after",
            self.assertEqual(d.get_size(key), i)
            #print "done"

    #----------------------------------------

    def test06_Truncate(self):
        d = self.d
        if verbose:
            print '\n', '-=' * 30
            print "Running %s.test06_Truncate..." % self.__class__.__name__

        d.put("abcde", "ABCDE");
        num = d.truncate()
        self.assertGreaterEqual(num, 1, "truncate returned <= 0 on non-empty database")
        num = d.truncate()
        self.assertEqual(num, 0,
                "truncate on empty DB returned nonzero (%r)" % (num,))

    #----------------------------------------

    def test07_verify(self):
        # Verify bug solved in 4.7.3pre8
        self.d.close()
        d = db.DB(self.env)
        d.verify(self.filename)


    #----------------------------------------

    if db.version() >= (4, 6):
        def test08_exists(self) :
            self.d.put("abcde", "ABCDE")
            self.assertEqual(self.d.exists("abcde"), True,
                    "DB->exists() returns wrong value")
            self.assertEqual(self.d.exists("x"), False,
                    "DB->exists() returns wrong value")

    #----------------------------------------

    if db.version() >= (4, 7):
        def test_compact(self) :
            d = self.d
            self.assertEqual(0, d.compact(flags=db.DB_FREELIST_ONLY))
            self.assertEqual(0, d.compact(flags=db.DB_FREELIST_ONLY))
            d.put("abcde", "ABCDE");
            d.put("bcde", "BCDE");
            d.put("abc", "ABC");
            d.put("monty", "python");
            d.delete("abc")
            d.delete("bcde")
            d.compact(start='abcde', stop='monty', txn=None,
                    compact_fillpercent=42, compact_pages=1,
                    compact_timeout=50000000,
                    flags=db.DB_FREELIST_ONLY|db.DB_FREE_SPACE)

    #----------------------------------------

#----------------------------------------------------------------------


class BasicBTreeTestCase(BasicTestCase):
    dbtype = db.DB_BTREE


class BasicHashTestCase(BasicTestCase):
    dbtype = db.DB_HASH


class BasicBTreeWithThreadFlagTestCase(BasicTestCase):
    dbtype = db.DB_BTREE
    dbopenflags = db.DB_THREAD


class BasicHashWithThreadFlagTestCase(BasicTestCase):
    dbtype = db.DB_HASH
    dbopenflags = db.DB_THREAD


class BasicWithEnvTestCase(BasicTestCase):
    dbopenflags = db.DB_THREAD
    useEnv = 1
    envflags = db.DB_THREAD | db.DB_INIT_MPOOL | db.DB_INIT_LOCK

    #----------------------------------------

    def test09_EnvRemoveAndRename(self):
        if not self.env:
            return

        if verbose:
            print '\n', '-=' * 30
            print "Running %s.test09_EnvRemoveAndRename..." % self.__class__.__name__

        # can't rename or remove an open DB
        self.d.close()

        newname = self.filename + '.renamed'
        self.env.dbrename(self.filename, None, newname)
        self.env.dbremove(newname)

    #----------------------------------------

class BasicBTreeWithEnvTestCase(BasicWithEnvTestCase):
    dbtype = db.DB_BTREE


class BasicHashWithEnvTestCase(BasicWithEnvTestCase):
    dbtype = db.DB_HASH


#----------------------------------------------------------------------

class BasicTransactionTestCase(BasicTestCase):
    if (sys.version_info < (2, 7)) or ((sys.version_info >= (3, 0)) and
            (sys.version_info < (3, 2))) :
        def assertIn(self, a, b, msg=None) :
            return self.assertTrue(a in b, msg=msg)

    dbopenflags = db.DB_THREAD | db.DB_AUTO_COMMIT
    useEnv = 1
    envflags = (db.DB_THREAD | db.DB_INIT_MPOOL | db.DB_INIT_LOCK |
                db.DB_INIT_TXN)
    envsetflags = db.DB_AUTO_COMMIT


    def tearDown(self):
        self.txn.commit()
        BasicTestCase.tearDown(self)


    def populateDB(self):
        txn = self.env.txn_begin()
        BasicTestCase.populateDB(self, _txn=txn)

        self.txn = self.env.txn_begin()


    def test06_Transactions(self):
        d = self.d
        if verbose:
            print '\n', '-=' * 30
            print "Running %s.test06_Transactions..." % self.__class__.__name__

        self.assertEqual(d.get('new rec', txn=self.txn), None)
        d.put('new rec', 'this is a new record', self.txn)
        self.assertEqual(d.get('new rec', txn=self.txn),
                'this is a new record')
        self.txn.abort()
        self.assertEqual(d.get('new rec'), None)

        self.txn = self.env.txn_begin()

        self.assertEqual(d.get('new rec', txn=self.txn), None)
        d.put('new rec', 'this is a new record', self.txn)
        self.assertEqual(d.get('new rec', txn=self.txn),
                'this is a new record')
        self.txn.commit()
        self.assertEqual(d.get('new rec'), 'this is a new record')

        self.txn = self.env.txn_begin()
        c = d.cursor(self.txn)
        rec = c.first()
        count = 0
        while rec is not None:
            count = count + 1
            if verbose and count % 100 == 0:
                print rec
            rec = c.next()
        self.assertEqual(count, self._numKeys+1)

        c.close()                # Cursors *MUST* be closed before commit!
        self.txn.commit()

        # flush pending updates
        self.env.txn_checkpoint (0, 0, 0)

        statDict = self.env.log_stat(0);
        self.assertIn('magic', statDict)
        self.assertIn('version', statDict)
        self.assertIn('cur_file', statDict)
        self.assertIn('region_nowait', statDict)

        # must have at least one log file present:
        logs = self.env.log_archive(db.DB_ARCH_ABS | db.DB_ARCH_LOG)
        self.assertNotEqual(logs, None)
        for log in logs:
            if verbose:
                print 'log file: ' + log
            logs = self.env.log_archive(db.DB_ARCH_REMOVE)
            self.assertFalse(logs)

        self.txn = self.env.txn_begin()

    #----------------------------------------

    if db.version() >= (4, 6):
        def test08_exists(self) :
            txn = self.env.txn_begin()
            self.d.put("abcde", "ABCDE", txn=txn)
            txn.commit()
            txn = self.env.txn_begin()
            self.assertEqual(self.d.exists("abcde", txn=txn), True,
                    "DB->exists() returns wrong value")
            self.assertEqual(self.d.exists("x", txn=txn), False,
                    "DB->exists() returns wrong value")
            txn.abort()

    #----------------------------------------

    def test09_TxnTruncate(self):
        d = self.d
        if verbose:
            print '\n', '-=' * 30
            print "Running %s.test09_TxnTruncate..." % self.__class__.__name__

        d.put("abcde", "ABCDE");
        txn = self.env.txn_begin()
        num = d.truncate(txn)
        self.assertGreaterEqual(num, 1, "truncate returned <= 0 on non-empty database")
        num = d.truncate(txn)
        self.assertEqual(num, 0,
                "truncate on empty DB returned nonzero (%r)" % (num,))
        txn.commit()

    #----------------------------------------

    def test10_TxnLateUse(self):
        txn = self.env.txn_begin()
        txn.abort()
        try:
            txn.abort()
        except db.DBError, e:
            pass
        else:
            raise RuntimeError, "DBTxn.abort() called after DB_TXN no longer valid w/o an exception"

        txn = self.env.txn_begin()
        txn.commit()
        try:
            txn.commit()
        except db.DBError, e:
            pass
        else:
            raise RuntimeError, "DBTxn.commit() called after DB_TXN no longer valid w/o an exception"


    #----------------------------------------


    if db.version() >= (4, 4):
        def test_txn_name(self) :
            txn=self.env.txn_begin()
            self.assertEqual(txn.get_name(), "")
            txn.set_name("XXYY")
            self.assertEqual(txn.get_name(), "XXYY")
            txn.set_name("")
            self.assertEqual(txn.get_name(), "")
            txn.abort()

    #----------------------------------------


        def test_txn_set_timeout(self) :
            txn=self.env.txn_begin()
            txn.set_timeout(1234567, db.DB_SET_LOCK_TIMEOUT)
            txn.set_timeout(2345678, flags=db.DB_SET_TXN_TIMEOUT)
            txn.abort()

    #----------------------------------------

        def test_get_tx_max(self) :
            self.assertEqual(self.env.get_tx_max(), 30)

        def test_get_tx_timestamp(self) :
            self.assertEqual(self.env.get_tx_timestamp(), self._t)



class BTreeTransactionTestCase(BasicTransactionTestCase):
    dbtype = db.DB_BTREE

class HashTransactionTestCase(BasicTransactionTestCase):
    dbtype = db.DB_HASH



#----------------------------------------------------------------------

class BTreeRecnoTestCase(BasicTestCase):
    dbtype     = db.DB_BTREE
    dbsetflags = db.DB_RECNUM

    def test09_RecnoInBTree(self):
        d = self.d
        if verbose:
            print '\n', '-=' * 30
            print "Running %s.test09_RecnoInBTree..." % self.__class__.__name__

        rec = d.get(200)
        self.assertEqual(type(rec), type(()))
        self.assertEqual(len(rec), 2)
        if verbose:
            print "Record #200 is ", rec

        c = d.cursor()
        c.set('0200')
        num = c.get_recno()
        self.assertEqual(type(num), type(1))
        if verbose:
            print "recno of d['0200'] is ", num

        rec = c.current()
        self.assertEqual(c.set_recno(num), rec)

        c.close()



class BTreeRecnoWithThreadFlagTestCase(BTreeRecnoTestCase):
    dbopenflags = db.DB_THREAD

#----------------------------------------------------------------------

class BasicDUPTestCase(BasicTestCase):
    dbsetflags = db.DB_DUP

    def test10_DuplicateKeys(self):
        d = self.d
        if verbose:
            print '\n', '-=' * 30
            print "Running %s.test10_DuplicateKeys..." % \
                  self.__class__.__name__

        d.put("dup0", "before")
        for x in "The quick brown fox jumped over the lazy dog.".split():
            d.put("dup1", x)
        d.put("dup2", "after")

        data = d.get("dup1")
        self.assertEqual(data, "The")
        if verbose:
            print data

        c = d.cursor()
        rec = c.set("dup1")
        self.assertEqual(rec, ('dup1', 'The'))

        next_reg = c.next()
        self.assertEqual(next_reg, ('dup1', 'quick'))

        rec = c.set("dup1")
        count = c.count()
        self.assertEqual(count, 9)

        next_dup = c.next_dup()
        self.assertEqual(next_dup, ('dup1', 'quick'))

        rec = c.set('dup1')
        while rec is not None:
            if verbose:
                print rec
            rec = c.next_dup()

        c.set('dup1')
        rec = c.next_nodup()
        self.assertNotEqual(rec[0], 'dup1')
        if verbose:
            print rec

        c.close()



class BTreeDUPTestCase(BasicDUPTestCase):
    dbtype = db.DB_BTREE

class HashDUPTestCase(BasicDUPTestCase):
    dbtype = db.DB_HASH

class BTreeDUPWithThreadTestCase(BasicDUPTestCase):
    dbtype = db.DB_BTREE
    dbopenflags = db.DB_THREAD

class HashDUPWithThreadTestCase(BasicDUPTestCase):
    dbtype = db.DB_HASH
    dbopenflags = db.DB_THREAD


#----------------------------------------------------------------------

class BasicMultiDBTestCase(BasicTestCase):
    dbname = 'first'

    def otherType(self):
        if self.dbtype == db.DB_BTREE:
            return db.DB_HASH
        else:
            return db.DB_BTREE

    def test11_MultiDB(self):
        d1 = self.d
        if verbose:
            print '\n', '-=' * 30
            print "Running %s.test11_MultiDB..." % self.__class__.__name__

        d2 = db.DB(self.env)
        d2.open(self.filename, "second", self.dbtype,
                self.dbopenflags|db.DB_CREATE)
        d3 = db.DB(self.env)
        d3.open(self.filename, "third", self.otherType(),
                self.dbopenflags|db.DB_CREATE)

        for x in "The quick brown fox jumped over the lazy dog".split():
            d2.put(x, self.makeData(x))

        for x in string.ascii_letters:
            d3.put(x, x*70)

        d1.sync()
        d2.sync()
        d3.sync()
        d1.close()
        d2.close()
        d3.close()

        self.d = d1 = d2 = d3 = None

        self.d = d1 = db.DB(self.env)
        d1.open(self.filename, self.dbname, flags = self.dbopenflags)
        d2 = db.DB(self.env)
        d2.open(self.filename, "second",  flags = self.dbopenflags)
        d3 = db.DB(self.env)
        d3.open(self.filename, "third", flags = self.dbopenflags)

        c1 = d1.cursor()
        c2 = d2.cursor()
        c3 = d3.cursor()

        count = 0
        rec = c1.first()
        while rec is not None:
            count = count + 1
            if verbose and (count % 50) == 0:
                print rec
            rec = c1.next()
        self.assertEqual(count, self._numKeys)

        count = 0
        rec = c2.first()
        while rec is not None:
            count = count + 1
            if verbose:
                print rec
            rec = c2.next()
        self.assertEqual(count, 9)

        count = 0
        rec = c3.first()
        while rec is not None:
            count = count + 1
            if verbose:
                print rec
            rec = c3.next()
        self.assertEqual(count, len(string.ascii_letters))


        c1.close()
        c2.close()
        c3.close()

        d2.close()
        d3.close()



# Strange things happen if you try to use Multiple DBs per file without a
# DBEnv with MPOOL and LOCKing...

class BTreeMultiDBTestCase(BasicMultiDBTestCase):
    dbtype = db.DB_BTREE
    dbopenflags = db.DB_THREAD
    useEnv = 1
    envflags = db.DB_THREAD | db.DB_INIT_MPOOL | db.DB_INIT_LOCK

class HashMultiDBTestCase(BasicMultiDBTestCase):
    dbtype = db.DB_HASH
    dbopenflags = db.DB_THREAD
    useEnv = 1
    envflags = db.DB_THREAD | db.DB_INIT_MPOOL | db.DB_INIT_LOCK


class PrivateObject(unittest.TestCase) :
    def tearDown(self) :
        del self.obj

    def test01_DefaultIsNone(self) :
        self.assertEqual(self.obj.get_private(), None)

    def test02_assignment(self) :
        a = "example of private object"
        self.obj.set_private(a)
        b = self.obj.get_private()
        self.assertIs(a, b)  # Object identity

    def test03_leak_assignment(self) :
        a = "example of private object"
        refcount = sys.getrefcount(a)
        self.obj.set_private(a)
        self.assertEqual(refcount+1, sys.getrefcount(a))
        self.obj.set_private(None)
        self.assertEqual(refcount, sys.getrefcount(a))

    def test04_leak_GC(self) :
        a = "example of private object"
        refcount = sys.getrefcount(a)
        self.obj.set_private(a)
        self.obj = None
        self.assertEqual(refcount, sys.getrefcount(a))

class DBEnvPrivateObject(PrivateObject) :
    def setUp(self) :
        self.obj = db.DBEnv()

class DBPrivateObject(PrivateObject) :
    def setUp(self) :
        self.obj = db.DB()

class CrashAndBurn(unittest.TestCase) :
    #def test01_OpenCrash(self) :
    #    # See http://bugs.python.org/issue3307
    #    self.assertRaises(db.DBInvalidArgError, db.DB, None, 65535)

    if db.version() < (4, 8) :
        def test02_DBEnv_dealloc(self):
            # http://bugs.python.org/issue3885
            import gc
            self.assertRaises(db.DBInvalidArgError, db.DBEnv, ~db.DB_RPCCLIENT)
            gc.collect()


#----------------------------------------------------------------------
#----------------------------------------------------------------------

def test_suite():
    suite = unittest.TestSuite()

    suite.addTest(unittest.makeSuite(VersionTestCase))
    suite.addTest(unittest.makeSuite(BasicBTreeTestCase))
    suite.addTest(unittest.makeSuite(BasicHashTestCase))
    suite.addTest(unittest.makeSuite(BasicBTreeWithThreadFlagTestCase))
    suite.addTest(unittest.makeSuite(BasicHashWithThreadFlagTestCase))
    suite.addTest(unittest.makeSuite(BasicBTreeWithEnvTestCase))
    suite.addTest(unittest.makeSuite(BasicHashWithEnvTestCase))
    suite.addTest(unittest.makeSuite(BTreeTransactionTestCase))
    suite.addTest(unittest.makeSuite(HashTransactionTestCase))
    suite.addTest(unittest.makeSuite(BTreeRecnoTestCase))
    suite.addTest(unittest.makeSuite(BTreeRecnoWithThreadFlagTestCase))
    suite.addTest(unittest.makeSuite(BTreeDUPTestCase))
    suite.addTest(unittest.makeSuite(HashDUPTestCase))
    suite.addTest(unittest.makeSuite(BTreeDUPWithThreadTestCase))
    suite.addTest(unittest.makeSuite(HashDUPWithThreadTestCase))
    suite.addTest(unittest.makeSuite(BTreeMultiDBTestCase))
    suite.addTest(unittest.makeSuite(HashMultiDBTestCase))
    suite.addTest(unittest.makeSuite(DBEnvPrivateObject))
    suite.addTest(unittest.makeSuite(DBPrivateObject))
    suite.addTest(unittest.makeSuite(CrashAndBurn))

    return suite


if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



"""Make the custom certificate and private key files used by test_ssl
and friends."""

import os
import shutil
import sys
import tempfile
from subprocess import *

req_template = """
    [req]
    distinguished_name     = req_distinguished_name
    x509_extensions        = req_x509_extensions
    prompt                 = no

    [req_distinguished_name]
    C                      = XY
    L                      = Castle Anthrax
    O                      = Python Software Foundation
    CN                     = {hostname}

    [req_x509_extensions]
    subjectAltName         = @san

    [san]
    DNS.1 = {hostname}
    {extra_san}

    [dir_sect]
    C                      = XY
    L                      = Castle Anthrax
    O                      = Python Software Foundation
    CN                     = dirname example

    [princ_name]
    realm = EXP:0, GeneralString:KERBEROS.REALM
    principal_name = EXP:1, SEQUENCE:principal_seq

    [principal_seq]
    name_type = EXP:0, INTEGER:1
    name_string = EXP:1, SEQUENCE:principals

    [principals]
    princ1 = GeneralString:username

    [ ca ]
    default_ca      = CA_default

    [ CA_default ]
    dir = cadir
    database  = $dir/index.txt
    crlnumber = $dir/crl.txt
    default_md = sha256
    default_days = 3600
    default_crl_days = 3600
    certificate = pycacert.pem
    private_key = pycakey.pem
    serial    = $dir/serial
    RANDFILE  = $dir/.rand

    policy          = policy_match

    [ policy_match ]
    countryName             = match
    stateOrProvinceName     = optional
    organizationName        = match
    organizationalUnitName  = optional
    commonName              = supplied
    emailAddress            = optional

    [ policy_anything ]
    countryName   = optional
    stateOrProvinceName = optional
    localityName    = optional
    organizationName  = optional
    organizationalUnitName  = optional
    commonName    = supplied
    emailAddress    = optional


    [ v3_ca ]

    subjectKeyIdentifier=hash
    authorityKeyIdentifier=keyid:always,issuer
    basicConstraints = CA:true

    """

here = os.path.abspath(os.path.dirname(__file__))


def make_cert_key(hostname, sign=False, extra_san='',
                  ext='req_x509_extensions_full', key='rsa:3072'):
    print("creating cert for " + hostname)
    tempnames = []
    for i in range(3):
        with tempfile.NamedTemporaryFile(delete=False) as f:
            tempnames.append(f.name)
    req_file, cert_file, key_file = tempnames
    try:
        req = req_template.format(hostname=hostname, extra_san=extra_san)
        with open(req_file, 'w') as f:
            f.write(req)
        args = ['req', '-new', '-days', '3650', '-nodes',
                '-newkey', 'rsa:1024', '-keyout', key_file,
                '-config', req_file]
        if sign:
            with tempfile.NamedTemporaryFile(delete=False) as f:
                tempnames.append(f.name)
                reqfile = f.name
            args += ['-out', reqfile ]

        else:
            args += ['-x509', '-out', cert_file ]
        check_call(['openssl'] + args)

        if sign:
            args = ['ca', '-config', req_file, '-out', cert_file, '-outdir', 'cadir',
                    '-policy', 'policy_anything', '-batch', '-infiles', reqfile ]
            check_call(['openssl'] + args)


        with open(cert_file, 'r') as f:
            cert = f.read()
        with open(key_file, 'r') as f:
            key = f.read()
        return cert, key
    finally:
        for name in tempnames:
            os.remove(name)

TMP_CADIR = 'cadir'

def unmake_ca():
    shutil.rmtree(TMP_CADIR)

def make_ca():
    os.mkdir(TMP_CADIR)
    with open(os.path.join('cadir','index.txt'),'a+') as f:
        pass # empty file
    with open(os.path.join('cadir','crl.txt'),'a+') as f:
        f.write("00")
    with open(os.path.join('cadir','index.txt.attr'),'w+') as f:
        f.write('unique_subject = no')

    with tempfile.NamedTemporaryFile("w") as t:
        t.write(req_template.format(hostname='our-ca-server', extra_san=''))
        t.flush()
        with tempfile.NamedTemporaryFile() as f:
            args = ['req', '-new', '-days', '3650', '-extensions', 'v3_ca', '-nodes',
                    '-newkey', 'rsa:3072', '-keyout', 'pycakey.pem',
                    '-out', f.name,
                    '-subj', '/C=XY/L=Castle Anthrax/O=Python Software Foundation CA/CN=our-ca-server']
            check_call(['openssl'] + args)
            args = ['ca', '-config', t.name, '-create_serial',
                    '-out', 'pycacert.pem', '-batch', '-outdir', TMP_CADIR,
                    '-keyfile', 'pycakey.pem', '-days', '3650',
                    '-selfsign', '-extensions', 'v3_ca', '-infiles', f.name ]
            check_call(['openssl'] + args)
            args = ['ca', '-config', t.name, '-gencrl', '-out', 'revocation.crl']
            check_call(['openssl'] + args)

if __name__ == '__main__':
    os.chdir(here)
    cert, key = make_cert_key('localhost')
    with open('ssl_cert.pem', 'w') as f:
        f.write(cert)
    with open('ssl_key.pem', 'w') as f:
        f.write(key)
    print("password protecting ssl_key.pem in ssl_key.passwd.pem")
    check_call(['openssl','rsa','-in','ssl_key.pem','-out','ssl_key.passwd.pem','-des3','-passout','pass:somepass'])
    check_call(['openssl','rsa','-in','ssl_key.pem','-out','keycert.passwd.pem','-des3','-passout','pass:somepass'])

    with open('keycert.pem', 'w') as f:
        f.write(key)
        f.write(cert)

    with open('keycert.passwd.pem', 'a+') as f:
        f.write(cert)

    # For certificate matching tests
    make_ca()
    cert, key = make_cert_key('fakehostname')
    with open('keycert2.pem', 'w') as f:
        f.write(key)
        f.write(cert)

    cert, key = make_cert_key('localhost', True)
    with open('keycert3.pem', 'w') as f:
        f.write(key)
        f.write(cert)

    cert, key = make_cert_key('fakehostname', True)
    with open('keycert4.pem', 'w') as f:
        f.write(key)
        f.write(cert)

    extra_san = [
        'otherName.1 = 1.2.3.4;UTF8:some other identifier',
        'otherName.2 = 1.3.6.1.5.2.2;SEQUENCE:princ_name',
        'email.1 = user@example.org',
        'DNS.2 = www.example.org',
        # GEN_X400
        'dirName.1 = dir_sect',
        # GEN_EDIPARTY
        'URI.1 = https://www.python.org/',
        'IP.1 = 127.0.0.1',
        'IP.2 = ::1',
        'RID.1 = 1.2.3.4.5',
    ]

    cert, key = make_cert_key('allsans', extra_san='\n'.join(extra_san))
    with open('allsans.pem', 'w') as f:
        f.write(key)
        f.write(cert)

    unmake_ca()
    print("\n\nPlease change the values in test_ssl.py, test_parse_cert function related to notAfter,notBefore and serialNumber")
    check_call(['openssl','x509','-in','keycert.pem','-dates','-serial','-noout'])
